#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from timeit import Timer, time

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

combinations = ['png',
                'png8',
                'png8:m=o',
                'png8:m=h',
                'png8:m=o:t=0',
                'png8:m=o:t=1',
                'png8:m=o:t=2',
                'png8:m=h:t=0',
                'png8:m=h:t=1',
                'png8:m=h:t=2',
                'png:z=1',
                'png:z=1:t=0',  # forces rbg, no a
                'png8:z=1',
                'png8:z=1:m=o',
                'png8:z=1:m=h',
                'png8:z=1:c=1',
                'png8:z=1:c=24',
                'png8:z=1:c=64',
                'png8:z=1:c=128',
                'png8:z=1:c=200',
                'png8:z=1:c=255',
                'png8:z=9:c=64',
                'png8:z=9:c=128',
                'png8:z=9:c=200',
                'png8:z=1:c=50:m=h',
                'png8:z=1:c=1:m=o',
                'png8:z=1:c=1:m=o:s=filtered',
                'png:z=1:s=filtered',
                'png:z=1:s=huff',
                'png:z=1:s=rle',
                'png8:m=h:g=2.0',
                'png8:m=h:g=1.0'
                ]

tiles = [
    'blank',
    'solid',
    'many_colors',
    'aerial_24'
]

iterations = 10


def do_encoding():

    global image

    results = {}
    sortable = {}

    def run(func, im, format, t):
        global image
        image = im
        start = time.time()
        set = t.repeat(iterations, 1)
        elapsed = (time.time() - start)
        min_ = min(set) * 1000
        avg = (sum(set) / len(set)) * 1000
        name = func.__name__ + ' ' + format
        results[name] = [min_, avg, elapsed * 1000, name, len(func())]
        sortable[name] = [min_]

    if 'blank' in tiles:
        def blank():
            return eval('image.tostring("%s")' % c)
        blank_im = mapnik.Image(512, 512)
        for c in combinations:
            t = Timer(blank)
            run(blank, blank_im, c, t)

    if 'solid' in tiles:
        def solid():
            return eval('image.tostring("%s")' % c)
        solid_im = mapnik.Image(512, 512)
        solid_im.fill(mapnik.Color("#f2efe9"))
        for c in combinations:
            t = Timer(solid)
            run(solid, solid_im, c, t)

    if 'many_colors' in tiles:
        def many_colors():
            return eval('image.tostring("%s")' % c)
        # lots of colors: http://tile.osm.org/13/4194/2747.png
        many_colors_im = mapnik.Image.open('../data/images/13_4194_2747.png')
        for c in combinations:
            t = Timer(many_colors)
            run(many_colors, many_colors_im, c, t)

    if 'aerial_24' in tiles:
        def aerial_24():
            return eval('image.tostring("%s")' % c)
        aerial_24_im = mapnik.Image.open('../data/images/12_654_1580.png')
        for c in combinations:
            t = Timer(aerial_24)
            run(aerial_24, aerial_24_im, c, t)

    for key, value in sorted(sortable.items(), key=lambda i: (i[1], i[0])):
        s = results[key]
        min_ = str(s[0])[:6]
        avg = str(s[1])[:6]
        elapsed = str(s[2])[:6]
        name = s[3]
        size = s[4]
        print(
            'min: %sms | avg: %sms | total: %sms | len: %s <-- %s' %
            (min_, avg, elapsed, size, name))


if __name__ == "__main__":
    setup()
    do_encoding()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



# coding=utf8
import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if 'csv' in mapnik.DatasourceCache.plugin_names():
    def test_marker_ellipse_render1():
        m = mapnik.Map(256, 256)
        mapnik.load_map(m, '../data/good_maps/marker_ellipse_transform.xml')
        m.zoom_all()
        im = mapnik.Image(m.width, m.height)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-marker-ellipse-render1.png'
        expected = 'images/support/mapnik-marker-ellipse-render1.png'
        im.save(actual, 'png32')
        if os.environ.get('UPDATE'):
            im.save(expected, 'png32')
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'test/python_tests/' + expected))

    def test_marker_ellipse_render2():
        m = mapnik.Map(256, 256)
        mapnik.load_map(m, '../data/good_maps/marker_ellipse_transform2.xml')
        m.zoom_all()
        im = mapnik.Image(m.width, m.height)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-marker-ellipse-render2.png'
        expected = 'images/support/mapnik-marker-ellipse-render2.png'
        im.save(actual, 'png32')
        if os.environ.get('UPDATE'):
            im.save(expected, 'png32')
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'test/python_tests/' + expected))

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



# coding=utf8
import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if 'sqlite' in mapnik.DatasourceCache.plugin_names():

    # the negative buffer on the layer should
    # override the postive map buffer leading
    # only one point to be rendered in the map
    def test_layer_buffer_size_1():
        m = mapnik.Map(512, 512)
        eq_(m.buffer_size, 0)
        mapnik.load_map(m, '../data/good_maps/layer_buffer_size_reduction.xml')
        eq_(m.buffer_size, 256)
        eq_(m.layers[0].buffer_size, -150)
        m.zoom_all()
        im = mapnik.Image(m.width, m.height)
        mapnik.render(m, im)
        actual = '/tmp/mapnik-layer-buffer-size.png'
        expected = 'images/support/mapnik-layer-buffer-size.png'
        im.save(actual, "png32")
        expected_im = mapnik.Image.open(expected)
        eq_(im.tostring('png32'),
            expected_im.tostring('png32'),
            'failed comparing actual (%s) and expected (%s)' % (actual,
                                                                'tests/python_tests/' + expected))


if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

import fnmatch
import os
import shutil
from subprocess import PIPE, Popen

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_shapeindex():
    # first copy shapefiles to tmp directory
    source_dir = '../data/shp/'
    working_dir = '/tmp/mapnik-shp-tmp/'
    if os.path.exists(working_dir):
        shutil.rmtree(working_dir)
    shutil.copytree(source_dir, working_dir)
    matches = []
    for root, dirnames, filenames in os.walk('%s' % source_dir):
        for filename in fnmatch.filter(filenames, '*.shp'):
            matches.append(os.path.join(root, filename))
    for shp in matches:
        source_file = os.path.join(
            source_dir, os.path.relpath(
                shp, source_dir))
        dest_file = os.path.join(working_dir, os.path.relpath(shp, source_dir))
        ds = mapnik.Shapefile(file=source_file)
        count = 0
        fs = ds.featureset()
        try:
            while (fs.next()):
                count = count + 1
        except StopIteration:
            pass
        stdin, stderr = Popen(
            'shapeindex %s' %
            dest_file, shell=True, stdout=PIPE, stderr=PIPE).communicate()
        ds2 = mapnik.Shapefile(file=dest_file)
        count2 = 0
        fs = ds.featureset()
        try:
            while (fs.next()):
                count2 = count2 + 1
        except StopIteration:
            pass
        eq_(count, count2)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



# encoding: utf8

from nose.tools import eq_

import mapnik

from .utilities import run_all

try:
    import json
except ImportError:
    import simplejson as json

chars = [
    {
        "name": "single_quote",
        "test": "string with ' quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \' quote"}}'
    },
    {
        "name": "escaped_single_quote",
        "test": "string with \' quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \' quote"}}'
    },
    {
        "name": "double_quote",
        "test": 'string with " quote',
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\" quote"}}'
    },
    {
        "name": "double_quote2",
        "test": "string with \" quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\" quote"}}'
    },
    {
        "name": "reverse_solidus",  # backslash
        "test": "string with \\ quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\\ quote"}}'
    },
    {
        "name": "solidus",  # forward slash
        "test": "string with / quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with / quote"}}'
    },
    {
        "name": "backspace",
        "test": "string with \b quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\b quote"}}'
    },
    {
        "name": "formfeed",
        "test": "string with \f quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\f quote"}}'
    },
    {
        "name": "newline",
        "test": "string with \n quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\n quote"}}'
    },
    {
        "name": "carriage_return",
        "test": "string with \r quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\r quote"}}'
    },
    {
        "name": "horiztonal_tab",
        "test": "string with \t quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\t quote"}}'
    },
    # remainder are c++ reserved, but not json
    {
        "name": "vert_tab",
        "test": "string with \v quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\u000b quote"}}'
    },
    {
        "name": "alert",
        "test": "string with \a quote",
        "json": '{"type":"Feature","id":1,"geometry":null,"properties":{"name":"string with \\u0007 quote"}}'
    }
]

ctx = mapnik.Context()
ctx.push('name')


def test_char_escaping():
    for char in chars:
        feat = mapnik.Feature(ctx, 1)
        expected = char['test']
        feat["name"] = expected
        eq_(feat["name"], expected)
        # confirm the python json module
        # is working as we would expect
        pyjson2 = json.loads(char['json'])
        eq_(pyjson2['properties']['name'], expected)
        # confirm our behavior is the same as python json module
        # for the original string
        geojson_feat_string = feat.to_geojson()
        eq_(
            geojson_feat_string,
            char['json'],
            "Mapnik's json escaping is not to spec: actual(%s) and expected(%s) for %s" %
            (geojson_feat_string,
             char['json'],
                char['name']))
        # and the round tripped string
        pyjson = json.loads(geojson_feat_string)
        eq_(pyjson['properties']['name'], expected)

if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



import glob
import os
from subprocess import Popen, PIPE
from distutils import sysconfig

Import('env')

def call(cmd, silent=True):
    stdin, stderr = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE).communicate()
    if not stderr:
        return stdin.strip()
    elif not silent:
        print stderr


prefix = env['PREFIX']
target_path = os.path.normpath(sysconfig.get_python_lib() + os.path.sep + env['MAPNIK_NAME'])

py_env = env.Clone()

py_env.Append(CPPPATH = sysconfig.get_python_inc())

py_env.Append(CPPDEFINES = env['LIBMAPNIK_DEFINES'])

py_env['LIBS'] = [env['MAPNIK_NAME'],'libboost_python']

link_all_libs = env['LINKING'] == 'static' or env['RUNTIME_LINK'] == 'static'

# even though boost_thread is no longer used in mapnik core
# we need to link in for boost_python to avoid missing symbol: _ZN5boost6detail12get_tss_dataEPKv / boost::detail::get_tss_data
py_env.AppendUnique(LIBS = 'boost_thread%s' % env['BOOST_APPEND'])

if link_all_libs:
    py_env.AppendUnique(LIBS=env['LIBMAPNIK_LIBS'])

# note: on linux -lrt must be linked after thread to avoid: undefined symbol: clock_gettime
if env['RUNTIME_LINK'] == 'static' and env['PLATFORM'] == 'Linux':
    py_env.AppendUnique(LIBS='rt')

# TODO - do solaris/fedora need direct linking too?
python_link_flag = ''
if env['PLATFORM'] == 'Darwin':
    python_link_flag = '-undefined dynamic_lookup'

paths = '''
"""Configuration paths of Mapnik fonts and input plugins (auto-generated by SCons)."""

from os.path import normpath,join,dirname

mapniklibpath = '%s'
mapniklibpath = normpath(join(dirname(__file__),mapniklibpath))
'''

paths += "inputpluginspath = join(mapniklibpath,'input')\n"

if env['SYSTEM_FONTS']:
    paths += "fontscollectionpath = normpath('%s')\n" % env['SYSTEM_FONTS']
else:
    paths += "fontscollectionpath = join(mapniklibpath,'fonts')\n"

paths += "__all__ = [mapniklibpath,inputpluginspath,fontscollectionpath]\n"

if not os.path.exists(env['MAPNIK_NAME']):
    os.mkdir(env['MAPNIK_NAME'])

file('mapnik/paths.py','w').write(paths % (env['MAPNIK_LIB_DIR']))

# force open perms temporarily so that `sudo scons install`
# does not later break simple non-install non-sudo rebuild
try:
    os.chmod('mapnik/paths.py',0666)
except: pass

# install the shared object beside the module directory
sources = glob.glob('src/*.cpp')

if 'install' in COMMAND_LINE_TARGETS:
    # install the core mapnik python files, including '__init__.py'
    init_files = glob.glob('mapnik/*.py')
    if 'mapnik/paths.py' in init_files:
        init_files.remove('mapnik/paths.py')
    init_module = env.Install(target_path, init_files)
    env.Alias(target='install', source=init_module)
    # fix perms and install the custom generated 'paths.py'
    targetp = os.path.join(target_path,'paths.py')
    env.Alias("install", targetp)
    # use env.Command rather than env.Install
    # to enable setting proper perms on `paths.py`
    env.Command( targetp, 'mapnik/paths.py',
        [
        Copy("$TARGET","$SOURCE"),
        Chmod("$TARGET", 0644),
        ])

if 'uninstall' not in COMMAND_LINE_TARGETS:
    if env['HAS_CAIRO']:
        py_env.Append(CPPPATH = env['CAIRO_CPPPATHS'])
        py_env.Append(CPPDEFINES = '-DHAVE_CAIRO')
        if link_all_libs:
            py_env.Append(LIBS=env['CAIRO_ALL_LIBS'])

    if env['HAS_PYCAIRO']:
        py_env.Append(CPPDEFINES = '-DHAVE_PYCAIRO')
        py_env.Append(CPPPATH = env['PYCAIRO_PATHS'])

py_env.Append(LINKFLAGS=python_link_flag)
py_env.AppendUnique(LIBS='mapnik-json')
py_env.AppendUnique(LIBS='mapnik-wkt')

_mapnik = py_env.LoadableModule('mapnik/_mapnik', sources, LDMODULEPREFIX='', LDMODULESUFFIX='.so')

Depends(_mapnik, env.subst('../../src/%s' % env['MAPNIK_LIB_NAME']))
Depends(_mapnik, env.subst('../../src/json/libmapnik-json${LIBSUFFIX}'))
Depends(_mapnik, env.subst('../../src/wkt/libmapnik-wkt${LIBSUFFIX}'))

if 'uninstall' not in COMMAND_LINE_TARGETS:
    pymapniklib = env.Install(target_path,_mapnik)
    py_env.Alias(target='install',source=pymapniklib)

env['create_uninstall_target'](env, target_path)



#!/usr/bin/env python

import atexit
import os
import re
import sys
import time
from binascii import hexlify
from subprocess import PIPE, Popen

from nose.tools import assert_almost_equal, eq_

import mapnik

from .utilities import execution_path, run_all, side_by_side_image

MAPNIK_TEST_DBNAME = 'mapnik-tmp-pgraster-test-db'
POSTGIS_TEMPLATE_DBNAME = 'template_postgis'
DEBUG_OUTPUT = False


def log(msg):
    if DEBUG_OUTPUT:
        print(msg)


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def call(cmd, silent=False):
    stdin, stderr = Popen(cmd, shell=True, stdout=PIPE,
                          stderr=PIPE).communicate()
    stdin = stdin.decode()
    stderr = stderr.decode()
    if not stderr:
        return stdin.strip()
    elif not silent and 'error' in stderr.lower() \
            or 'not found' in stderr.lower() \
            or 'could not connect' in stderr.lower() \
            or 'bad connection' in stderr.lower() \
            or 'not recognized as an internal' in stderr.lower():
        raise RuntimeError(stderr.strip())


def psql_can_connect():
    """Test ability to connect to a postgis template db with no options.

    Basically, to run these tests your user must have full read
    access over unix sockets without supplying a password. This
    keeps these tests simple and focused on postgis not on postgres
    auth issues.
    """
    try:
        call('psql %s -c "select postgis_version()"' % POSTGIS_TEMPLATE_DBNAME)
        return True
    except RuntimeError:
        print('Notice: skipping pgraster tests (connection)')
        return False


def psql_run(cmd):
    cmd = 'psql --set ON_ERROR_STOP=1 %s -c "%s"' % \
        (MAPNIK_TEST_DBNAME, cmd.replace('"', '\\"'))
    log('DEBUG: running ' + cmd)
    call(cmd)


def raster2pgsql_on_path():
    """Test for presence of raster2pgsql on the user path.

    We require this program to load test data into a temporarily database.
    """
    try:
        call('raster2pgsql')
        return True
    except RuntimeError:
        print('Notice: skipping pgraster tests (raster2pgsql)')
        return False


def createdb_and_dropdb_on_path():
    """Test for presence of dropdb/createdb on user path.

    We require these programs to setup and teardown the testing db.
    """
    try:
        call('createdb --help')
        call('dropdb --help')
        return True
    except RuntimeError:
        print('Notice: skipping pgraster tests (createdb/dropdb)')
        return False


def postgis_setup():
    call('dropdb %s' % MAPNIK_TEST_DBNAME, silent=True)
    call(
        'createdb -T %s %s' %
        (POSTGIS_TEMPLATE_DBNAME,
         MAPNIK_TEST_DBNAME),
        silent=False)


def postgis_takedown():
    pass
    # fails as the db is in use: https://github.com/mapnik/mapnik/issues/960
    #call('dropdb %s' % MAPNIK_TEST_DBNAME)


def import_raster(filename, tabname, tilesize, constraint, overview):
    log('tile: ' + tilesize + ' constraints: ' + str(constraint)
        + ' overviews: ' + overview)
    cmd = 'raster2pgsql -Y -I -q'
    if constraint:
        cmd += ' -C'
    if tilesize:
        cmd += ' -t ' + tilesize
    if overview:
        cmd += ' -l ' + overview
    cmd += ' %s %s | psql --set ON_ERROR_STOP=1 -q %s' % (
        os.path.abspath(os.path.normpath(filename)), tabname, MAPNIK_TEST_DBNAME)
    log('Import call: ' + cmd)
    call(cmd)


def drop_imported(tabname, overview):
    psql_run('DROP TABLE IF EXISTS "' + tabname + '";')
    if overview:
        for of in overview.split(','):
            psql_run('DROP TABLE IF EXISTS "o_' + of + '_' + tabname + '";')


def compare_images(expected, im):
    expected = os.path.join(
        os.path.dirname(expected),
        os.path.basename(expected).replace(
            ':',
            '_'))
    if not os.path.exists(expected) or os.environ.get('UPDATE'):
        print('generating expected image %s' % expected)
        im.save(expected, 'png32')
    expected_im = mapnik.Image.open(expected)
    diff = expected.replace('.png', '-diff.png')
    if len(im.tostring("png32")) != len(expected_im.tostring("png32")):
        compared = side_by_side_image(expected_im, im)
        compared.save(diff)
        assert False, 'images do not match, check diff at %s' % diff
    else:
        if os.path.exists(diff):
            os.unlink(diff)
    return True

if 'pgraster' in mapnik.DatasourceCache.plugin_names() \
        and createdb_and_dropdb_on_path() \
        and psql_can_connect() \
        and raster2pgsql_on_path():

        # initialize test database
    postgis_setup()

    # [old]dataraster.tif, 2283x1913 int16 single-band
    # dataraster-small.tif, 457x383 int16 single-band
    def _test_dataraster_16bsi_rendering(lbl, overview, rescale, clip):
        if rescale:
            lbl += ' Sc'
        if clip:
            lbl += ' Cl'
        ds = mapnik.PgRaster(dbname=MAPNIK_TEST_DBNAME, table='"dataRaster"',
                             band=1, use_overviews=1 if overview else 0,
                             prescale_rasters=rescale, clip_rasters=clip)
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['rid'], 1)
        lyr = mapnik.Layer('dataraster_16bsi')
        lyr.datasource = ds
        expenv = mapnik.Box2d(-14637, 3903178, 1126863, 4859678)
        env = lyr.envelope()
        # As the input size is a prime number both horizontally
        # and vertically, we expect the extent of the overview
        # tables to be a pixel wider than the original, whereas
        # the pixel size in geographical units depends on the
        # overview factor. So we start with the original pixel size
        # as base scale and multiply by the overview factor.
        # NOTE: the overview table extent only grows north and east
        pixsize = 500  # see gdalinfo dataraster.tif
        pixsize = 2497  # see gdalinfo dataraster-small.tif
        tol = pixsize * max(overview.split(',')) if overview else 0
        assert_almost_equal(env.minx, expenv.minx)
        assert_almost_equal(env.miny, expenv.miny, delta=tol)
        assert_almost_equal(env.maxx, expenv.maxx, delta=tol)
        assert_almost_equal(env.maxy, expenv.maxy)
        mm = mapnik.Map(256, 256)
        style = mapnik.Style()
        col = mapnik.RasterColorizer()
        col.default_mode = mapnik.COLORIZER_DISCRETE
        col.add_stop(0, mapnik.Color(0x40, 0x40, 0x40, 255))
        col.add_stop(10, mapnik.Color(0x80, 0x80, 0x80, 255))
        col.add_stop(20, mapnik.Color(0xa0, 0xa0, 0xa0, 255))
        sym = mapnik.RasterSymbolizer()
        sym.colorizer = col
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style.rules.append(rule)
        mm.append_style('foo', style)
        lyr.styles.append('foo')
        mm.layers.append(lyr)
        mm.zoom_to_box(expenv)
        im = mapnik.Image(mm.width, mm.height)
        t0 = time.time()  # we want wall time to include IO waits
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:full')
        # no data
        eq_(im.view(1, 1, 1, 1).tostring(), b'\x00\x00\x00\x00')
        eq_(im.view(255, 255, 1, 1).tostring(), b'\x00\x00\x00\x00')
        eq_(im.view(195, 116, 1, 1).tostring(), b'\x00\x00\x00\x00')
        # A0A0A0
        eq_(im.view(100, 120, 1, 1).tostring(), b'\xa0\xa0\xa0\xff')
        eq_(im.view(75, 80, 1, 1).tostring(), b'\xa0\xa0\xa0\xff')
        # 808080
        eq_(im.view(74, 170, 1, 1).tostring(), b'\x80\x80\x80\xff')
        eq_(im.view(30, 50, 1, 1).tostring(), b'\x80\x80\x80\xff')
        # 404040
        eq_(im.view(190, 70, 1, 1).tostring(), b'\x40\x40\x40\xff')
        eq_(im.view(140, 170, 1, 1).tostring(), b'\x40\x40\x40\xff')

        # Now zoom over a portion of the env (1/10)
        newenv = mapnik.Box2d(273663, 4024478, 330738, 4072303)
        mm.zoom_to_box(newenv)
        t0 = time.time()  # we want wall time to include IO waits
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:1/10')
        # nodata
        eq_(hexlify(im.view(255, 255, 1, 1).tostring()), b'00000000')
        eq_(hexlify(im.view(200, 254, 1, 1).tostring()), b'00000000')
        # A0A0A0
        eq_(hexlify(im.view(90, 232, 1, 1).tostring()), b'a0a0a0ff')
        eq_(hexlify(im.view(96, 245, 1, 1).tostring()), b'a0a0a0ff')
        # 808080
        eq_(hexlify(im.view(1, 1, 1, 1).tostring()), b'808080ff')
        eq_(hexlify(im.view(128, 128, 1, 1).tostring()), b'808080ff')
        # 404040
        eq_(hexlify(im.view(255, 0, 1, 1).tostring()), b'404040ff')

    def _test_dataraster_16bsi(lbl, tilesize, constraint, overview):
        import_raster(
            '../data/raster/dataraster-small.tif',
            'dataRaster',
            tilesize,
            constraint,
            overview)
        if constraint:
            lbl += ' C'
        if tilesize:
            lbl += ' T:' + tilesize
        if overview:
            lbl += ' O:' + overview
        for prescale in [0, 1]:
            for clip in [0, 1]:
                _test_dataraster_16bsi_rendering(lbl, overview, prescale, clip)
        drop_imported('dataRaster', overview)

    def test_dataraster_16bsi():
        # for tilesize in ['','256x256']:
        for tilesize in ['256x256']:
            for constraint in [0, 1]:
                # for overview in ['','4','2,16']:
                for overview in ['', '2']:
                    _test_dataraster_16bsi(
                        'data_16bsi', tilesize, constraint, overview)

    # river.tiff, RGBA 8BUI
    def _test_rgba_8bui_rendering(lbl, overview, rescale, clip):
        if rescale:
            lbl += ' Sc'
        if clip:
            lbl += ' Cl'
        ds = mapnik.PgRaster(dbname=MAPNIK_TEST_DBNAME, table='(select * from "River") foo',
                             use_overviews=1 if overview else 0,
                             prescale_rasters=rescale, clip_rasters=clip)
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['rid'], 1)
        lyr = mapnik.Layer('rgba_8bui')
        lyr.datasource = ds
        expenv = mapnik.Box2d(0, -210, 256, 0)
        env = lyr.envelope()
        # As the input size is a prime number both horizontally
        # and vertically, we expect the extent of the overview
        # tables to be a pixel wider than the original, whereas
        # the pixel size in geographical units depends on the
        # overview factor. So we start with the original pixel size
        # as base scale and multiply by the overview factor.
        # NOTE: the overview table extent only grows north and east
        pixsize = 1  # see gdalinfo river.tif
        tol = pixsize * max(overview.split(',')) if overview else 0
        assert_almost_equal(env.minx, expenv.minx)
        assert_almost_equal(env.miny, expenv.miny, delta=tol)
        assert_almost_equal(env.maxx, expenv.maxx, delta=tol)
        assert_almost_equal(env.maxy, expenv.maxy)
        mm = mapnik.Map(256, 256)
        style = mapnik.Style()
        sym = mapnik.RasterSymbolizer()
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style.rules.append(rule)
        mm.append_style('foo', style)
        lyr.styles.append('foo')
        mm.layers.append(lyr)
        mm.zoom_to_box(expenv)
        im = mapnik.Image(mm.width, mm.height)
        t0 = time.time()  # we want wall time to include IO waits
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:full')
        expected = 'images/support/pgraster/%s-%s-%s-%s-box1.png' % (
            lyr.name, lbl, overview, clip)
        compare_images(expected, im)
        # no data
        eq_(hexlify(im.view(3, 3, 1, 1).tostring()), b'00000000')
        eq_(hexlify(im.view(250, 250, 1, 1).tostring()), b'00000000')
        # full opaque river color
        eq_(hexlify(im.view(175, 118, 1, 1).tostring()), b'b9d8f8ff')
        # half-transparent pixel
        pxstr = hexlify(im.view(122, 138, 1, 1).tostring()).decode()
        apat = ".*(..)$"
        match = re.match(apat, pxstr)
        assert match, 'pixel ' + pxstr + ' does not match pattern ' + apat
        alpha = match.group(1)
        assert alpha != 'ff' and alpha != '00', \
            'unexpected full transparent/opaque pixel: ' + alpha

        # Now zoom over a portion of the env (1/10)
        newenv = mapnik.Box2d(166, -105, 191, -77)
        mm.zoom_to_box(newenv)
        t0 = time.time()  # we want wall time to include IO waits
        im = mapnik.Image(mm.width, mm.height)
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:1/10')
        expected = 'images/support/pgraster/%s-%s-%s-%s-box2.png' % (
            lyr.name, lbl, overview, clip)
        compare_images(expected, im)
        # no data
        eq_(hexlify(im.view(255, 255, 1, 1).tostring()), b'00000000')
        eq_(hexlify(im.view(200, 40, 1, 1).tostring()), b'00000000')
        # full opaque river color
        eq_(hexlify(im.view(100, 168, 1, 1).tostring()), b'b9d8f8ff')
        # half-transparent pixel
        pxstr = hexlify(im.view(122, 138, 1, 1).tostring()).decode()
        apat = ".*(..)$"
        match = re.match(apat, pxstr)
        assert match, 'pixel ' + pxstr + ' does not match pattern ' + apat
        alpha = match.group(1)
        assert alpha != 'ff' and alpha != '00', \
            'unexpected full transparent/opaque pixel: ' + alpha

    def _test_rgba_8bui(lbl, tilesize, constraint, overview):
        import_raster(
            '../data/raster/river.tiff',
            'River',
            tilesize,
            constraint,
            overview)
        if constraint:
            lbl += ' C'
        if tilesize:
            lbl += ' T:' + tilesize
        if overview:
            lbl += ' O:' + overview
        for prescale in [0, 1]:
            for clip in [0, 1]:
                _test_rgba_8bui_rendering(lbl, overview, prescale, clip)
        drop_imported('River', overview)

    def test_rgba_8bui():
        for tilesize in ['', '16x16']:
            for constraint in [0, 1]:
                for overview in ['2']:
                    _test_rgba_8bui(
                        'rgba_8bui', tilesize, constraint, overview)

    # nodata-edge.tif, RGB 8BUI
    def _test_rgb_8bui_rendering(lbl, tnam, overview, rescale, clip):
        if rescale:
            lbl += ' Sc'
        if clip:
            lbl += ' Cl'
        ds = mapnik.PgRaster(dbname=MAPNIK_TEST_DBNAME, table=tnam,
                             use_overviews=1 if overview else 0,
                             prescale_rasters=rescale, clip_rasters=clip)
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['rid'], 1)
        lyr = mapnik.Layer('rgba_8bui')
        lyr.datasource = ds
        expenv = mapnik.Box2d(-12329035.7652168, 4508650.39854396,
                              -12328653.0279471, 4508957.34625536)
        env = lyr.envelope()
        # As the input size is a prime number both horizontally
        # and vertically, we expect the extent of the overview
        # tables to be a pixel wider than the original, whereas
        # the pixel size in geographical units depends on the
        # overview factor. So we start with the original pixel size
        # as base scale and multiply by the overview factor.
        # NOTE: the overview table extent only grows north and east
        pixsize = 2  # see gdalinfo nodata-edge.tif
        tol = pixsize * max(overview.split(',')) if overview else 0
        assert_almost_equal(env.minx, expenv.minx, places=0)
        assert_almost_equal(env.miny, expenv.miny, delta=tol)
        assert_almost_equal(env.maxx, expenv.maxx, delta=tol)
        assert_almost_equal(env.maxy, expenv.maxy, places=0)
        mm = mapnik.Map(256, 256)
        style = mapnik.Style()
        sym = mapnik.RasterSymbolizer()
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style.rules.append(rule)
        mm.append_style('foo', style)
        lyr.styles.append('foo')
        mm.layers.append(lyr)
        mm.zoom_to_box(expenv)
        im = mapnik.Image(mm.width, mm.height)
        t0 = time.time()  # we want wall time to include IO waits
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:full')
        expected = 'images/support/pgraster/%s-%s-%s-%s-%s-box1.png' % (
            lyr.name, tnam, lbl, overview, clip)
        compare_images(expected, im)
        # no data
        eq_(hexlify(im.view(3, 16, 1, 1).tostring()), '00000000')
        eq_(hexlify(im.view(128, 16, 1, 1).tostring()), '00000000')
        eq_(hexlify(im.view(250, 16, 1, 1).tostring()), '00000000')
        eq_(hexlify(im.view(3, 240, 1, 1).tostring()), '00000000')
        eq_(hexlify(im.view(128, 240, 1, 1).tostring()), '00000000')
        eq_(hexlify(im.view(250, 240, 1, 1).tostring()), '00000000')
        # dark brown
        eq_(hexlify(im.view(174, 39, 1, 1).tostring()), 'c3a698ff')
        # dark gray
        eq_(hexlify(im.view(195, 132, 1, 1).tostring()), '575f62ff')
        # Now zoom over a portion of the env (1/10)
        newenv = mapnik.Box2d(-12329035.7652168, 4508926.651484220,
                              -12328997.49148983, 4508957.34625536)
        mm.zoom_to_box(newenv)
        t0 = time.time()  # we want wall time to include IO waits
        im = mapnik.Image(mm.width, mm.height)
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:1/10')
        expected = 'images/support/pgraster/%s-%s-%s-%s-%s-box2.png' % (
            lyr.name, tnam, lbl, overview, clip)
        compare_images(expected, im)
        # no data
        eq_(hexlify(im.view(3, 16, 1, 1).tostring()), '00000000')
        eq_(hexlify(im.view(128, 16, 1, 1).tostring()), '00000000')
        eq_(hexlify(im.view(250, 16, 1, 1).tostring()), '00000000')
        # black
        eq_(hexlify(im.view(3, 42, 1, 1).tostring()), '000000ff')
        eq_(hexlify(im.view(3, 134, 1, 1).tostring()), '000000ff')
        eq_(hexlify(im.view(3, 244, 1, 1).tostring()), '000000ff')
        # gray
        eq_(hexlify(im.view(135, 157, 1, 1).tostring()), '4e555bff')
        # brown
        eq_(hexlify(im.view(195, 223, 1, 1).tostring()), 'f2cdbaff')

    def _test_rgb_8bui(lbl, tilesize, constraint, overview):
        tnam = 'nodataedge'
        import_raster(
            '../data/raster/nodata-edge.tif',
            tnam,
            tilesize,
            constraint,
            overview)
        if constraint:
            lbl += ' C'
        if tilesize:
            lbl += ' T:' + tilesize
        if overview:
            lbl += ' O:' + overview
        for prescale in [0, 1]:
            for clip in [0, 1]:
                _test_rgb_8bui_rendering(lbl, tnam, overview, prescale, clip)
        #drop_imported(tnam, overview)

    def test_rgb_8bui():
        for tilesize in ['64x64']:
            for constraint in [1]:
                for overview in ['']:
                    _test_rgb_8bui('rgb_8bui', tilesize, constraint, overview)

    def _test_grayscale_subquery(lbl, pixtype, value):
        #
        #      3   8   13
        #    +---+---+---+
        #  3 | v | v | v |  NOTE: writes different color
        #    +---+---+---+        in 13,8 and 8,13
        #  8 | v | v | a |
        #    +---+---+---+
        # 13 | v | b | v |
        #    +---+---+---+
        #
        val_a = int(value / 3)
        val_b = val_a * 2
        sql = "(select 3 as i, " \
              " ST_SetValues(" \
              "  ST_SetValues(" \
              "   ST_AsRaster(" \
              "    ST_MakeEnvelope(0,0,14,14), " \
              "    1.0, -1.0, '%s', %s" \
              "   ), " \
              "   11, 6, 4, 5, %s::float8" \
              "  )," \
              "  6, 11, 5, 4, %s::float8" \
              " ) as \"R\"" \
              ") as foo" % (pixtype, value, val_a, val_b)
        rescale = 0
        clip = 0
        if rescale:
            lbl += ' Sc'
        if clip:
            lbl += ' Cl'
        ds = mapnik.PgRaster(dbname=MAPNIK_TEST_DBNAME, table=sql,
                             raster_field='"R"', use_overviews=1,
                             prescale_rasters=rescale, clip_rasters=clip)
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['i'], 3)
        lyr = mapnik.Layer('grayscale_subquery')
        lyr.datasource = ds
        expenv = mapnik.Box2d(0, 0, 14, 14)
        env = lyr.envelope()
        assert_almost_equal(env.minx, expenv.minx, places=0)
        assert_almost_equal(env.miny, expenv.miny, places=0)
        assert_almost_equal(env.maxx, expenv.maxx, places=0)
        assert_almost_equal(env.maxy, expenv.maxy, places=0)
        mm = mapnik.Map(15, 15)
        style = mapnik.Style()
        sym = mapnik.RasterSymbolizer()
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style.rules.append(rule)
        mm.append_style('foo', style)
        lyr.styles.append('foo')
        mm.layers.append(lyr)
        mm.zoom_to_box(expenv)
        im = mapnik.Image(mm.width, mm.height)
        t0 = time.time()  # we want wall time to include IO waits
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:full')
        expected = 'images/support/pgraster/%s-%s-%s-%s.png' % (
            lyr.name, lbl, pixtype, value)
        compare_images(expected, im)
        h = format(value, '02x')
        hex_v = h + h + h + 'ff'
        hex_v = hex_v.encode()
        h = format(val_a, '02x')
        hex_a = h + h + h + 'ff'
        hex_a = hex_a.encode()
        h = format(val_b, '02x')
        hex_b = h + h + h + 'ff'
        hex_b = hex_b.encode()
        eq_(hexlify(im.view(3, 3, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(8, 3, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(13, 3, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(3, 8, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(8, 8, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(13, 8, 1, 1).tostring()), hex_a)
        eq_(hexlify(im.view(3, 13, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(8, 13, 1, 1).tostring()), hex_b)
        eq_(hexlify(im.view(13, 13, 1, 1).tostring()), hex_v)

    def test_grayscale_2bui_subquery():
        _test_grayscale_subquery('grayscale_2bui_subquery', '2BUI', 3)

    def test_grayscale_4bui_subquery():
        _test_grayscale_subquery('grayscale_4bui_subquery', '4BUI', 15)

    def test_grayscale_8bui_subquery():
        _test_grayscale_subquery('grayscale_8bui_subquery', '8BUI', 63)

    def test_grayscale_8bsi_subquery():
        # NOTE: we're using a positive integer because Mapnik
        #       does not support negative data values anyway
        _test_grayscale_subquery('grayscale_8bsi_subquery', '8BSI', 69)

    def test_grayscale_16bui_subquery():
        _test_grayscale_subquery('grayscale_16bui_subquery', '16BUI', 126)

    def test_grayscale_16bsi_subquery():
        # NOTE: we're using a positive integer because Mapnik
        #       does not support negative data values anyway
        _test_grayscale_subquery('grayscale_16bsi_subquery', '16BSI', 144)

    def test_grayscale_32bui_subquery():
        _test_grayscale_subquery('grayscale_32bui_subquery', '32BUI', 255)

    def test_grayscale_32bsi_subquery():
        # NOTE: we're using a positive integer because Mapnik
        #       does not support negative data values anyway
        _test_grayscale_subquery('grayscale_32bsi_subquery', '32BSI', 129)

    def _test_data_subquery(lbl, pixtype, value):
        #
        #      3   8   13
        #    +---+---+---+
        #  3 | v | v | v |  NOTE: writes different values
        #    +---+---+---+        in 13,8 and 8,13
        #  8 | v | v | a |
        #    +---+---+---+
        # 13 | v | b | v |
        #    +---+---+---+
        #
        val_a = value / 3
        val_b = val_a * 2
        sql = "(select 3 as i, " \
              " ST_SetValues(" \
              "  ST_SetValues(" \
              "   ST_AsRaster(" \
              "    ST_MakeEnvelope(0,0,14,14), " \
              "    1.0, -1.0, '%s', %s" \
              "   ), " \
              "   11, 6, 5, 5, %s::float8" \
              "  )," \
              "  6, 11, 5, 5, %s::float8" \
              " ) as \"R\"" \
              ") as foo" % (pixtype, value, val_a, val_b)
        overview = ''
        rescale = 0
        clip = 0
        if rescale:
            lbl += ' Sc'
        if clip:
            lbl += ' Cl'
        ds = mapnik.PgRaster(dbname=MAPNIK_TEST_DBNAME, table=sql,
                             raster_field='R', use_overviews=0 if overview else 0,
                             band=1, prescale_rasters=rescale, clip_rasters=clip)
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['i'], 3)
        lyr = mapnik.Layer('data_subquery')
        lyr.datasource = ds
        expenv = mapnik.Box2d(0, 0, 14, 14)
        env = lyr.envelope()
        assert_almost_equal(env.minx, expenv.minx, places=0)
        assert_almost_equal(env.miny, expenv.miny, places=0)
        assert_almost_equal(env.maxx, expenv.maxx, places=0)
        assert_almost_equal(env.maxy, expenv.maxy, places=0)
        mm = mapnik.Map(15, 15)
        style = mapnik.Style()
        col = mapnik.RasterColorizer()
        col.default_mode = mapnik.COLORIZER_DISCRETE
        col.add_stop(val_a, mapnik.Color(0xff, 0x00, 0x00, 255))
        col.add_stop(val_b, mapnik.Color(0x00, 0xff, 0x00, 255))
        col.add_stop(value, mapnik.Color(0x00, 0x00, 0xff, 255))
        sym = mapnik.RasterSymbolizer()
        sym.colorizer = col
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style.rules.append(rule)
        mm.append_style('foo', style)
        lyr.styles.append('foo')
        mm.layers.append(lyr)
        mm.zoom_to_box(expenv)
        im = mapnik.Image(mm.width, mm.height)
        t0 = time.time()  # we want wall time to include IO waits
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:full')
        expected = 'images/support/pgraster/%s-%s-%s-%s.png' % (
            lyr.name, lbl, pixtype, value)
        compare_images(expected, im)

    def test_data_2bui_subquery():
        _test_data_subquery('data_2bui_subquery', '2BUI', 3)

    def test_data_4bui_subquery():
        _test_data_subquery('data_4bui_subquery', '4BUI', 15)

    def test_data_8bui_subquery():
        _test_data_subquery('data_8bui_subquery', '8BUI', 63)

    def test_data_8bsi_subquery():
        # NOTE: we're using a positive integer because Mapnik
        #       does not support negative data values anyway
        _test_data_subquery('data_8bsi_subquery', '8BSI', 69)

    def test_data_16bui_subquery():
        _test_data_subquery('data_16bui_subquery', '16BUI', 126)

    def test_data_16bsi_subquery():
        # NOTE: we're using a positive integer because Mapnik
        #       does not support negative data values anyway
        _test_data_subquery('data_16bsi_subquery', '16BSI', 135)

    def test_data_32bui_subquery():
        _test_data_subquery('data_32bui_subquery', '32BUI', 255)

    def test_data_32bsi_subquery():
        # NOTE: we're using a positive integer because Mapnik
        #       does not support negative data values anyway
        _test_data_subquery('data_32bsi_subquery', '32BSI', 264)

    def test_data_32bf_subquery():
        _test_data_subquery('data_32bf_subquery', '32BF', 450)

    def test_data_64bf_subquery():
        _test_data_subquery('data_64bf_subquery', '64BF', 3072)

    def _test_rgba_subquery(lbl, pixtype, r, g, b, a, g1, b1):
        #
        #      3   8   13
        #    +---+---+---+
        #  3 | v | v | h |  NOTE: writes different alpha
        #    +---+---+---+        in 13,8 and 8,13
        #  8 | v | v | a |
        #    +---+---+---+
        # 13 | v | b | v |
        #    +---+---+---+
        #
        sql = "(select 3 as i, " \
              " ST_SetValues(" \
              "  ST_SetValues(" \
              "   ST_AddBand(" \
              "    ST_AddBand(" \
              "     ST_AddBand(" \
              "      ST_AsRaster(" \
              "       ST_MakeEnvelope(0,0,14,14), " \
              "       1.0, -1.0, '%s', %s" \
              "      )," \
              "      '%s', %d::float" \
              "     ), " \
              "     '%s', %d::float" \
              "    ), " \
              "    '%s', %d::float" \
              "   ), " \
              "   2, 11, 6, 4, 5, %s::float8" \
              "  )," \
              "  3, 6, 11, 5, 4, %s::float8" \
              " ) as r" \
              ") as foo" % (
                  pixtype,
                  r,
                  pixtype,
                  g,
                  pixtype,
                  b,
                  pixtype,
                  a,
                  g1,
                  b1)
        overview = ''
        rescale = 0
        clip = 0
        if rescale:
            lbl += ' Sc'
        if clip:
            lbl += ' Cl'
        ds = mapnik.PgRaster(dbname=MAPNIK_TEST_DBNAME, table=sql,
                             raster_field='r', use_overviews=0 if overview else 0,
                             prescale_rasters=rescale, clip_rasters=clip)
        fs = ds.featureset()
        feature = fs.next()
        eq_(feature['i'], 3)
        lyr = mapnik.Layer('rgba_subquery')
        lyr.datasource = ds
        expenv = mapnik.Box2d(0, 0, 14, 14)
        env = lyr.envelope()
        assert_almost_equal(env.minx, expenv.minx, places=0)
        assert_almost_equal(env.miny, expenv.miny, places=0)
        assert_almost_equal(env.maxx, expenv.maxx, places=0)
        assert_almost_equal(env.maxy, expenv.maxy, places=0)
        mm = mapnik.Map(15, 15)
        style = mapnik.Style()
        sym = mapnik.RasterSymbolizer()
        rule = mapnik.Rule()
        rule.symbols.append(sym)
        style.rules.append(rule)
        mm.append_style('foo', style)
        lyr.styles.append('foo')
        mm.layers.append(lyr)
        mm.zoom_to_box(expenv)
        im = mapnik.Image(mm.width, mm.height)
        t0 = time.time()  # we want wall time to include IO waits
        mapnik.render(mm, im)
        lap = time.time() - t0
        log('T ' + str(lap) + ' -- ' + lbl + ' E:full')
        expected = 'images/support/pgraster/%s-%s-%s-%s-%s-%s-%s-%s-%s.png' % (
            lyr.name, lbl, pixtype, r, g, b, a, g1, b1)
        compare_images(expected, im)
        hex_v = format(r << 24 | g << 16 | b << 8 | a, '08x').encode()
        hex_a = format(r << 24 | g1 << 16 | b << 8 | a, '08x').encode()
        hex_b = format(r << 24 | g << 16 | b1 << 8 | a, '08x').encode()
        eq_(hexlify(im.view(3, 3, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(8, 3, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(13, 3, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(3, 8, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(8, 8, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(13, 8, 1, 1).tostring()), hex_a)
        eq_(hexlify(im.view(3, 13, 1, 1).tostring()), hex_v)
        eq_(hexlify(im.view(8, 13, 1, 1).tostring()), hex_b)
        eq_(hexlify(im.view(13, 13, 1, 1).tostring()), hex_v)

    def test_rgba_8bui_subquery():
        _test_rgba_subquery(
            'rgba_8bui_subquery',
            '8BUI',
            255,
            0,
            0,
            255,
            255,
            255)

    # def test_rgba_16bui_subquery():
    #  _test_rgba_subquery('rgba_16bui_subquery', '16BUI', 65535, 0, 0, 65535, 65535, 65535)

    # def test_rgba_32bui_subquery():
    #  _test_rgba_subquery('rgba_32bui_subquery', '32BUI')

    atexit.register(postgis_takedown)


def enabled(tname):
    enabled = len(sys.argv) < 2 or tname in sys.argv
    if not enabled:
        print("Skipping " + tname + " as not explicitly enabled")
    return enabled

if __name__ == "__main__":
    setup()
    fail = run_all(eval(x)
                   for x in dir() if x.startswith("test_") and enabled(x))
    exit(fail)



#!/usr/bin/env python
import os
import sqlite3
import sys
import threading

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all

PYTHON3 = sys.version_info[0] == 3


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

NUM_THREADS = 10
TOTAL = 245


def create_ds(test_db, table):
    ds = mapnik.SQLite(file=test_db, table=table)
    ds.all_features()
    del ds

if 'sqlite' in mapnik.DatasourceCache.plugin_names():

    def test_rtree_creation():
        test_db = '../data/sqlite/world.sqlite'
        index = test_db + '.index'
        table = 'world_merc'

        if os.path.exists(index):
            os.unlink(index)

        threads = []
        for i in range(NUM_THREADS):
            t = threading.Thread(target=create_ds, args=(test_db, table))
            t.start()
            threads.append(t)

        for i in threads:
            i.join()

        eq_(os.path.exists(index), True)
        conn = sqlite3.connect(index)
        cur = conn.cursor()
        try:
            cur.execute(
                "Select count(*) from idx_%s_GEOMETRY" %
                table.replace(
                    "'", ""))
            conn.commit()
            eq_(cur.fetchone()[0], TOTAL)
        except sqlite3.OperationalError:
            # don't worry about testing # of index records if
            # python's sqlite module does not support rtree
            pass
        cur.close()
        conn.close()

        ds = mapnik.SQLite(file=test_db, table=table)
        fs = list(ds.all_features())
        del ds
        eq_(len(fs), TOTAL)
        os.unlink(index)
        ds = mapnik.SQLite(file=test_db, table=table, use_spatial_index=False)
        fs = list(ds.all_features())
        del ds
        eq_(len(fs), TOTAL)
        eq_(os.path.exists(index), False)

        ds = mapnik.SQLite(file=test_db, table=table, use_spatial_index=True)
        fs = list(ds.all_features())
        # TODO - this loop is not releasing something
        # because it causes the unlink below to fail on windows
        # as the file is still open
        # for feat in fs:
        #    query = mapnik.Query(feat.envelope())
        #    selected = ds.features(query)
        #    eq_(len(selected.features)>=1,True)
        del ds

        eq_(os.path.exists(index), True)
        os.unlink(index)

    test_rtree_creation.requires_data = True

    def test_geometry_round_trip():
        test_db = '/tmp/mapnik-sqlite-point.db'
        ogr_metadata = True

        # create test db
        conn = sqlite3.connect(test_db)
        cur = conn.cursor()
        cur.execute('''
             CREATE TABLE IF NOT EXISTS point_table
             (id INTEGER PRIMARY KEY AUTOINCREMENT, geometry BLOB, name varchar)
             ''')
        # optional: but nice if we want to read with ogr
        if ogr_metadata:
            cur.execute('''CREATE TABLE IF NOT EXISTS geometry_columns (
                        f_table_name VARCHAR,
                        f_geometry_column VARCHAR,
                        geometry_type INTEGER,
                        coord_dimension INTEGER,
                        srid INTEGER,
                        geometry_format VARCHAR )''')
            cur.execute('''INSERT INTO geometry_columns
                        (f_table_name, f_geometry_column, geometry_format,
                        geometry_type, coord_dimension, srid) VALUES
                        ('point_table','geometry','WKB', 1, 1, 4326)''')
        conn.commit()
        cur.close()

        # add a point as wkb (using mapnik) to match how an ogr created db
        # looks
        x = -122  # longitude
        y = 48  # latitude
        wkt = 'POINT(%s %s)' % (x, y)
        # little endian wkb (mapnik will auto-detect and ready either little or
        # big endian (XDR))
        wkb = mapnik.Geometry.from_wkt(wkt).to_wkb(mapnik.wkbByteOrder.NDR)
        values = (None, sqlite3.Binary(wkb), "test point")
        cur = conn.cursor()
        cur.execute(
            '''INSERT into "point_table" (id,geometry,name) values (?,?,?)''',
            values)
        conn.commit()
        cur.close()
        conn.close()

        def make_wkb_point(x, y):
            import struct
            byteorder = 1  # little endian
            endianess = ''
            if byteorder == 1:
                endianess = '<'
            else:
                endianess = '>'
            geom_type = 1  # for a point
            return struct.pack('%sbldd' % endianess,
                               byteorder, geom_type, x, y)

        # confirm the wkb matches a manually formed wkb
        wkb2 = make_wkb_point(x, y)
        eq_(wkb, wkb2)

        # ensure we can read this data back out properly with mapnik
        ds = mapnik.Datasource(
            **{'type': 'sqlite', 'file': test_db, 'table': 'point_table'})
        fs = ds.featureset()
        feat = fs.next()
        eq_(feat.id(), 1)
        eq_(feat['name'], 'test point')
        geom = feat.geometry
        eq_(geom.to_wkt(), 'POINT(-122 48)')
        del ds

        # ensure it matches data read with just sqlite
        conn = sqlite3.connect(test_db)
        cur = conn.cursor()
        cur.execute('''SELECT * from point_table''')
        conn.commit()
        result = cur.fetchone()
        cur.close()
        feat_id = result[0]
        eq_(feat_id, 1)
        name = result[2]
        eq_(name, 'test point')
        geom_wkb_blob = result[1]
        if not PYTHON3:
            geom_wkb_blob = str(geom_wkb_blob)
        eq_(geom_wkb_blob, geom.to_wkb(mapnik.wkbByteOrder.NDR))
        new_geom = mapnik.Geometry.from_wkb(geom_wkb_blob)
        eq_(new_geom.to_wkt(), geom.to_wkt())
        conn.close()
        os.unlink(test_db)

if __name__ == "__main__":
    setup()
    returncode = run_all(eval(x) for x in dir() if x.startswith("test_"))
    exit(returncode)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import traceback

from nose.plugins.errorclass import ErrorClass, ErrorClassPlugin
from nose.tools import assert_almost_equal

import mapnik

PYTHON3 = sys.version_info[0] == 3
READ_FLAGS = 'rb' if PYTHON3 else 'r'
if PYTHON3:
    xrange = range

HERE = os.path.dirname(__file__)


def execution_path(filename):
    return os.path.join(os.path.dirname(
        sys._getframe(1).f_code.co_filename), filename)


class Todo(Exception):
    pass


class TodoPlugin(ErrorClassPlugin):
    name = "todo"

    todo = ErrorClass(Todo, label='TODO', isfailure=False)


def contains_word(word, bytestring_):
    """
    Checks that a bytestring contains a given word. len(bytestring) should be
    a multiple of len(word).

    >>> contains_word("abcd", "abcd"*5)
    True

    >>> contains_word("ab", "ba"*5)
    False

    >>> contains_word("ab", "ab"*5+"a")
    Traceback (most recent call last):
    ...
    AssertionError: len(bytestring_) not multiple of len(word)
    """
    n = len(word)
    assert len(bytestring_) % n == 0, "len(bytestring_) not multiple of len(word)"
    chunks = [bytestring_[i:i + n] for i in xrange(0, len(bytestring_), n)]
    return word in chunks


def pixel2channels(pixel):
    alpha = (pixel >> 24) & 0xff
    red = pixel & 0xff
    green = (pixel >> 8) & 0xff
    blue = (pixel >> 16) & 0xff
    return red, green, blue, alpha


def pixel2rgba(pixel):
    return 'rgba(%s,%s,%s,%s)' % pixel2channels(pixel)


def get_unique_colors(im):
    pixels = []
    for x in range(im.width()):
        for y in range(im.height()):
            pixel = im.get_pixel(x, y)
            if pixel not in pixels:
                pixels.append(pixel)
    pixels = sorted(pixels)
    return list(map(pixel2rgba, pixels))


def run_all(iterable):
    failed = 0
    for test in iterable:
        try:
            test()
            sys.stderr.write("\x1b[32m✓ \x1b[m" + test.__name__ + "\x1b[m\n")
        except:
            exc_type, exc_value, exc_tb = sys.exc_info()
            failed += 1
            sys.stderr.write("\x1b[31m✘ \x1b[m" + test.__name__ + "\x1b[m\n")
            for mline in traceback.format_exception_only(exc_type, exc_value):
                for line in mline.rstrip().split("\n"):
                    sys.stderr.write("  \x1b[31m" + line + "\x1b[m\n")
            sys.stderr.write("  Traceback:\n")
            for mline in traceback.format_tb(exc_tb):
                for line in mline.rstrip().split("\n"):
                    if not 'utilities.py' in line and not 'trivial.py' in line and not line.strip() == 'test()':
                        sys.stderr.write("  " + line + "\n")
        sys.stderr.flush()
    return failed


def side_by_side_image(left_im, right_im):
    width = left_im.width() + 1 + right_im.width()
    height = max(left_im.height(), right_im.height())
    im = mapnik.Image(width, height)
    im.composite(left_im, mapnik.CompositeOp.src_over, 1.0, 0, 0)
    if width > 80:
        im.composite(
            mapnik.Image.open(
                HERE +
                '/images/expected.png'),
            mapnik.CompositeOp.difference,
            1.0,
            0,
            0)
    im.composite(
        right_im,
        mapnik.CompositeOp.src_over,
        1.0,
        left_im.width() + 1,
        0)
    if width > 80:
        im.composite(
            mapnik.Image.open(
                HERE +
                '/images/actual.png'),
            mapnik.CompositeOp.difference,
            1.0,
            left_im.width() +
            1,
            0)
    return im


def assert_box2d_almost_equal(a, b, msg=None):
    msg = msg or ("%r != %r" % (a, b))
    assert_almost_equal(a.minx, b.minx, msg=msg)
    assert_almost_equal(a.maxx, b.maxx, msg=msg)
    assert_almost_equal(a.miny, b.miny, msg=msg)
    assert_almost_equal(a.maxy, b.maxy, msg=msg)



#!/usr/bin/env python

import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_introspect_symbolizers():
    # create a symbolizer
    p = mapnik.PointSymbolizer()
    p.file = "../data/images/dummy.png"
    p.allow_overlap = True
    p.opacity = 0.5

    eq_(p.allow_overlap, True)
    eq_(p.opacity, 0.5)
    eq_(p.filename, '../data/images/dummy.png')

    # make sure the defaults
    # are what we think they are
    eq_(p.allow_overlap, True)
    eq_(p.opacity, 0.5)
    eq_(p.filename, '../data/images/dummy.png')

    # contruct objects to hold it
    r = mapnik.Rule()
    r.symbols.append(p)
    s = mapnik.Style()
    s.rules.append(r)
    m = mapnik.Map(0, 0)
    m.append_style('s', s)

    # try to figure out what is
    # in the map and make sure
    # style is there and the same

    s2 = m.find_style('s')
    rules = s2.rules
    eq_(len(rules), 1)
    r2 = rules[0]
    syms = r2.symbols
    eq_(len(syms), 1)

    # TODO here, we can do...
    sym = syms[0]
    p2 = sym.extract()
    assert isinstance(p2, mapnik.PointSymbolizer)

    eq_(p2.allow_overlap, True)
    eq_(p2.opacity, 0.5)
    eq_(p2.filename, '../data/images/dummy.png')

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import pickle

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


#def test_color_pickle():
#    c = mapnik.Color('blue')
#    eq_(pickle.loads(pickle.dumps(c)), c)
#    c = mapnik.Color(0, 64, 128)
#    eq_(pickle.loads(pickle.dumps(c)), c)
#    c = mapnik.Color(0, 64, 128, 192)
#    eq_(pickle.loads(pickle.dumps(c)), c)


#def test_envelope_pickle():
#    e = mapnik.Box2d(100, 100, 200, 200)
#    eq_(pickle.loads(pickle.dumps(e)), e)


def test_parameters_pickle():
    params = mapnik.Parameters()
    params.append(mapnik.Parameter('oh', str('yeah')))

    params2 = pickle.loads(pickle.dumps(params, pickle.HIGHEST_PROTOCOL))

    eq_(params[0][0], params2[0][0])
    eq_(params[0][1], params2[0][1])

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
import math
import sys

from nose.tools import assert_almost_equal, eq_

import mapnik

from .utilities import assert_box2d_almost_equal, run_all

PYTHON3 = sys.version_info[0] == 3
if PYTHON3:
    xrange = range

# Tests that exercise map projections.


def test_normalizing_definition():
    p = mapnik.Projection('+init=epsg:4326')
    expanded = p.expanded()
    eq_('+proj=longlat' in expanded, True)


# Trac Ticket #128
def test_wgs84_inverse_forward():
    p = mapnik.Projection('+init=epsg:4326')

    c = mapnik.Coord(3.01331418311, 43.3333092669)
    e = mapnik.Box2d(-122.54345245, 45.12312553, 68.2335581353, 48.231231233)

    # It appears that the y component changes very slightly, is this OK?
    # so we test for 'almost equal float values'

    assert_almost_equal(p.inverse(c).y, c.y)
    assert_almost_equal(p.inverse(c).x, c.x)

    assert_almost_equal(p.forward(c).y, c.y)
    assert_almost_equal(p.forward(c).x, c.x)

    assert_almost_equal(p.inverse(e).center().y, e.center().y)
    assert_almost_equal(p.inverse(e).center().x, e.center().x)

    assert_almost_equal(p.forward(e).center().y, e.center().y)
    assert_almost_equal(p.forward(e).center().x, e.center().x)

    assert_almost_equal(c.inverse(p).y, c.y)
    assert_almost_equal(c.inverse(p).x, c.x)

    assert_almost_equal(c.forward(p).y, c.y)
    assert_almost_equal(c.forward(p).x, c.x)

    assert_almost_equal(e.inverse(p).center().y, e.center().y)
    assert_almost_equal(e.inverse(p).center().x, e.center().x)

    assert_almost_equal(e.forward(p).center().y, e.center().y)
    assert_almost_equal(e.forward(p).center().x, e.center().x)


def wgs2merc(lon, lat):
    x = lon * 20037508.34 / 180
    y = math.log(math.tan((90 + lat) * math.pi / 360)) / (math.pi / 180)
    y = y * 20037508.34 / 180
    return [x, y]


def merc2wgs(x, y):
    x = (x / 20037508.34) * 180
    y = (y / 20037508.34) * 180
    y = 180 / math.pi * \
        (2 * math.atan(math.exp(y * math.pi / 180)) - math.pi / 2)
    if x > 180:
        x = 180
    if x < -180:
        x = -180
    if y > 85.0511:
        y = 85.0511
    if y < -85.0511:
        y = -85.0511
    return [x, y]

# echo -109 37 | cs2cs -f "%.10f" +init=epsg:4326 +to +init=epsg:3857
#-12133824.4964668211    4439106.7872505859 0.0000000000

# todo
# benchmarks
# better well known detection
# better srs matching with strip/trim
# python copy to avoid crash


def test_proj_transform_between_init_and_literal():
    one = mapnik.Projection('+init=epsg:4326')
    two = mapnik.Projection('+init=epsg:3857')
    tr1 = mapnik.ProjTransform(one, two)
    tr1b = mapnik.ProjTransform(two, one)
    wgs84 = '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs'
    merc = '+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0.0 +k=1.0 +units=m +nadgrids=@null +wktext +no_defs +over'
    src = mapnik.Projection(wgs84)
    dest = mapnik.Projection(merc)
    tr2 = mapnik.ProjTransform(src, dest)
    tr2b = mapnik.ProjTransform(dest, src)
    for x in xrange(-180, 180, 10):
        for y in xrange(-60, 60, 10):
            coord = mapnik.Coord(x, y)
            merc_coord1 = tr1.forward(coord)
            merc_coord2 = tr1b.backward(coord)
            merc_coord3 = tr2.forward(coord)
            merc_coord4 = tr2b.backward(coord)
            eq_(math.fabs(merc_coord1.x - merc_coord1.x) < 1, True)
            eq_(math.fabs(merc_coord1.x - merc_coord2.x) < 1, True)
            eq_(math.fabs(merc_coord1.x - merc_coord3.x) < 1, True)
            eq_(math.fabs(merc_coord1.x - merc_coord4.x) < 1, True)
            eq_(math.fabs(merc_coord1.y - merc_coord1.y) < 1, True)
            eq_(math.fabs(merc_coord1.y - merc_coord2.y) < 1, True)
            eq_(math.fabs(merc_coord1.y - merc_coord3.y) < 1, True)
            eq_(math.fabs(merc_coord1.y - merc_coord4.y) < 1, True)
            lon_lat_coord1 = tr1.backward(merc_coord1)
            lon_lat_coord2 = tr1b.forward(merc_coord2)
            lon_lat_coord3 = tr2.backward(merc_coord3)
            lon_lat_coord4 = tr2b.forward(merc_coord4)
            eq_(math.fabs(coord.x - lon_lat_coord1.x) < 1, True)
            eq_(math.fabs(coord.x - lon_lat_coord2.x) < 1, True)
            eq_(math.fabs(coord.x - lon_lat_coord3.x) < 1, True)
            eq_(math.fabs(coord.x - lon_lat_coord4.x) < 1, True)
            eq_(math.fabs(coord.y - lon_lat_coord1.y) < 1, True)
            eq_(math.fabs(coord.y - lon_lat_coord2.y) < 1, True)
            eq_(math.fabs(coord.y - lon_lat_coord3.y) < 1, True)
            eq_(math.fabs(coord.y - lon_lat_coord4.y) < 1, True)


# Github Issue #2648
def test_proj_antimeridian_bbox():
    # this is logic from feature_style_processor::prepare_layer()
    PROJ_ENVELOPE_POINTS = 20  # include/mapnik/config.hpp

    prjGeog = mapnik.Projection('+init=epsg:4326')
    prjProj = mapnik.Projection('+init=epsg:2193')
    prj_trans_fwd = mapnik.ProjTransform(prjProj, prjGeog)
    prj_trans_rev = mapnik.ProjTransform(prjGeog, prjProj)

    # bad = mapnik.Box2d(-177.31453250437079, -62.33374815225163, 178.02778363316355, -24.584597490955804)
    better = mapnik.Box2d(-180.0, -62.33374815225163,
                          180.0, -24.584597490955804)

    buffered_query_ext = mapnik.Box2d(274000, 3087000, 3327000, 7173000)
    fwd_ext = prj_trans_fwd.forward(buffered_query_ext, PROJ_ENVELOPE_POINTS)
    assert_box2d_almost_equal(fwd_ext, better)

    # check the same logic works for .backward()
    ext = mapnik.Box2d(274000, 3087000, 3327000, 7173000)
    rev_ext = prj_trans_rev.backward(ext, PROJ_ENVELOPE_POINTS)
    assert_box2d_almost_equal(rev_ext, better)

    # checks for not being snapped (ie. not antimeridian)
    normal = mapnik.Box2d(148.766759749, -60.1222810238,
                          159.95484893, -24.9771195151)
    buffered_query_ext = mapnik.Box2d(274000, 3087000, 276000, 7173000)
    fwd_ext = prj_trans_fwd.forward(buffered_query_ext, PROJ_ENVELOPE_POINTS)
    assert_box2d_almost_equal(fwd_ext, normal)

    # check the same logic works for .backward()
    ext = mapnik.Box2d(274000, 3087000, 276000, 7173000)
    rev_ext = prj_trans_rev.backward(ext, PROJ_ENVELOPE_POINTS)
    assert_box2d_almost_equal(rev_ext, normal)


if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



# #!/usr/bin/env python
# # -*- coding: utf-8 -*-

# import os
# import math
# import mapnik
# import sys
# from utilities import execution_path, run_all
# from nose.tools import *

# def setup():
#     # All of the paths used are relative, if we run the tests
#     # from another directory we need to chdir()
#     os.chdir(execution_path('.'))

# class PointDatasource(mapnik.PythonDatasource):
#     def __init__(self):
#         super(PointDatasource, self).__init__(
#                 geometry_type = mapnik.DataGeometryType.Point,
#                 envelope = mapnik.Box2d(0,-10,100,110),
#                 data_type = mapnik.DataType.Vector
#         )

#     def features(self, query):
#         return mapnik.PythonDatasource.wkt_features(
#             keys = ('label',),
#             features = (
#                 ( 'POINT (5 6)', { 'label': 'foo-bar'} ),
#                 ( 'POINT (60 50)', { 'label': 'buzz-quux'} ),
#             )
#         )

# class ConcentricCircles(object):
#     def __init__(self, centre, bounds, step=1):
#         self.centre = centre
#         self.bounds = bounds
#         self.step = step

#     class Iterator(object):
#         def __init__(self, container):
#             self.container = container

#             centre = self.container.centre
#             bounds = self.container.bounds
#             step = self.container.step

#             self.radius = step

#         def next(self):
#             points = []
#             for alpha in xrange(0, 361, 5):
#                 x = math.sin(math.radians(alpha)) * self.radius + self.container.centre[0]
#                 y = math.cos(math.radians(alpha)) * self.radius + self.container.centre[1]
#                 points.append('%s %s' % (x,y))
#             circle = 'POLYGON ((' + ','.join(points) + '))'

#             # has the circle grown so large that the boundary is entirely within it?
#             tl = (self.container.bounds.maxx, self.container.bounds.maxy)
#             tr = (self.container.bounds.maxx, self.container.bounds.maxy)
#             bl = (self.container.bounds.minx, self.container.bounds.miny)
#             br = (self.container.bounds.minx, self.container.bounds.miny)
#             def within_circle(p):
#                 delta_x = p[0] - self.container.centre[0]
#                 delta_y = p[0] - self.container.centre[0]
# return delta_x*delta_x + delta_y*delta_y < self.radius*self.radius

#             if all(within_circle(p) for p in (tl,tr,bl,br)):
#                 raise StopIteration()

#             self.radius += self.container.step
#             return ( circle, { } )

#     def __iter__(self):
#         return ConcentricCircles.Iterator(self)

# class CirclesDatasource(mapnik.PythonDatasource):
#     def __init__(self, centre_x=-20, centre_y=0, step=10):
#         super(CirclesDatasource, self).__init__(
#                 geometry_type = mapnik.DataGeometryType.Polygon,
#                 envelope = mapnik.Box2d(-180, -90, 180, 90),
#                 data_type = mapnik.DataType.Vector
#         )

#         # note that the plugin loader will set all arguments to strings and will not try to parse them
#         centre_x = int(centre_x)
#         centre_y = int(centre_y)
#         step = int(step)

#         self.centre_x = centre_x
#         self.centre_y = centre_y
#         self.step = step

#     def features(self, query):
#         centre = (self.centre_x, self.centre_y)

#         return mapnik.PythonDatasource.wkt_features(
#             keys = (),
#             features = ConcentricCircles(centre, query.bbox, self.step)
#         )

# if 'python' in mapnik.DatasourceCache.plugin_names():
#     # make sure we can load from ourself as a module
#     sys.path.append(execution_path('.'))

#     def test_python_point_init():
#         ds = mapnik.Python(factory='python_plugin_test:PointDatasource')
#         e = ds.envelope()

#         assert_almost_equal(e.minx, 0, places=7)
#         assert_almost_equal(e.miny, -10, places=7)
#         assert_almost_equal(e.maxx, 100, places=7)
#         assert_almost_equal(e.maxy, 110, places=7)

#     def test_python_circle_init():
#         ds = mapnik.Python(factory='python_plugin_test:CirclesDatasource')
#         e = ds.envelope()

#         assert_almost_equal(e.minx, -180, places=7)
#         assert_almost_equal(e.miny, -90, places=7)
#         assert_almost_equal(e.maxx, 180, places=7)
#         assert_almost_equal(e.maxy, 90, places=7)

#     def test_python_circle_init_with_args():
#         ds = mapnik.Python(factory='python_plugin_test:CirclesDatasource', centre_x=40, centre_y=7)
#         e = ds.envelope()

#         assert_almost_equal(e.minx, -180, places=7)
#         assert_almost_equal(e.miny, -90, places=7)
#         assert_almost_equal(e.maxx, 180, places=7)
#         assert_almost_equal(e.maxy, 90, places=7)

#     def test_python_point_rendering():
#         m = mapnik.Map(512,512)
#         mapnik.load_map(m,'../data/python_plugin/python_point_datasource.xml')
#         m.zoom_all()
#         im = mapnik.Image(512,512)
#         mapnik.render(m,im)
#         actual = '/tmp/mapnik-python-point-render1.png'
#         expected = 'images/support/mapnik-python-point-render1.png'
#         im.save(actual)
#         expected_im = mapnik.Image.open(expected)
#         eq_(im.tostring('png32'),expected_im.tostring('png32'),
#                 'failed comparing actual (%s) and expected (%s)' % (actual,'tests/python_tests/'+ expected))

#     def test_python_circle_rendering():
#         m = mapnik.Map(512,512)
#         mapnik.load_map(m,'../data/python_plugin/python_circle_datasource.xml')
#         m.zoom_all()
#         im = mapnik.Image(512,512)
#         mapnik.render(m,im)
#         actual = '/tmp/mapnik-python-circle-render1.png'
#         expected = 'images/support/mapnik-python-circle-render1.png'
#         im.save(actual)
#         expected_im = mapnik.Image.open(expected)
#         eq_(im.tostring('png32'),expected_im.tostring('png32'),
#                 'failed comparing actual (%s) and expected (%s)' % (actual,'tests/python_tests/'+ expected))

# if __name__ == "__main__":
#     setup()
#     run_all(eval(x) for x in dir() if x.startswith("test_"))



import os

from nose.tools import eq_

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_clearing_image_data():
    im = mapnik.Image(256, 256)
    # make sure it equals itself
    bytes = im.tostring()
    eq_(im.tostring(), bytes)
    # set background, then clear
    im.fill(mapnik.Color('green'))
    eq_(im.tostring() != bytes, True)
    # clear image, should now equal original
    im.clear()
    eq_(im.tostring(), bytes)


def make_map():
    ds = mapnik.MemoryDatasource()
    context = mapnik.Context()
    context.push('Name')
    pixel_key = 1
    f = mapnik.Feature(context, pixel_key)
    f['Name'] = str(pixel_key)
    f.geometry = mapnik.Geometry.from_wkt(
        'POLYGON ((0 0, 0 256, 256 256, 256 0, 0 0))')
    ds.add_feature(f)
    s = mapnik.Style()
    r = mapnik.Rule()
    symb = mapnik.PolygonSymbolizer()
    r.symbols.append(symb)
    s.rules.append(r)
    lyr = mapnik.Layer('Places')
    lyr.datasource = ds
    lyr.styles.append('places_labels')
    width, height = 256, 256
    m = mapnik.Map(width, height)
    m.append_style('places_labels', s)
    m.layers.append(lyr)
    m.zoom_all()
    return m

if mapnik.has_grid_renderer():
    def test_clearing_grid_data():
        g = mapnik.Grid(256, 256)
        utf = g.encode()
        # make sure it equals itself
        eq_(g.encode(), utf)
        m = make_map()
        mapnik.render_layer(m, g, layer=0, fields=['__id__', 'Name'])
        eq_(g.encode() != utf, True)
        # clear grid, should now match original
        g.clear()
        eq_(g.encode(), utf)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import glob
import os
import platform
import shutil
import sys

import mapnik

# mapnik.logger.set_severity(mapnik.severity_type.None)
# mapnik.logger.set_severity(mapnik.severity_type.Debug)

try:
    import json
except ImportError:
    import simplejson as json

visual_output_dir = "/tmp/mapnik-visual-images"

defaults = {
    'status': True,
    'sizes': [(500, 100)],
    'scales': [1.0, 2.0],
    'agg': True,
    'cairo': mapnik.has_cairo(),
    'grid': mapnik.has_grid_renderer()
}

cairo_threshold = 10
agg_threshold = 0
grid_threshold = 5
if 'Linux' == platform.uname()[0]:
    # we assume if linux then you are running packaged cairo
    # which is older than the 1.12.14 version we used on OS X
    # to generate the expected images, so we'll rachet back the threshold
    # https://github.com/mapnik/mapnik/issues/1868
    cairo_threshold = 230
    agg_threshold = 12
    grid_threshold = 6


def render_cairo(m, output, scale_factor):
    mapnik.render_to_file(m, output, 'ARGB32', scale_factor)
    # open and re-save as png8 to save space
    new_im = mapnik.Image.open(output)
    new_im.save(output, 'png32')


def render_grid(m, output, scale_factor):
    grid = mapnik.Grid(m.width, m.height)
    mapnik.render_layer(m, grid, layer=0, scale_factor=scale_factor)
    utf1 = grid.encode('utf', resolution=4)
    open(output, 'wb').write(json.dumps(utf1, indent=1).encode())


def render_agg(m, output, scale_factor):
    mapnik.render_to_file(m, output, 'png32', scale_factor),

renderers = [
    {'name': 'agg',
     'render': render_agg,
     'compare': lambda actual, reference: compare(actual, reference, alpha=True),
     'threshold': agg_threshold,
     'filetype': 'png',
     'dir': 'images'
     },
    {'name': 'cairo',
     'render': render_cairo,
     'compare': lambda actual, reference: compare(actual, reference, alpha=False),
     'threshold': cairo_threshold,
     'filetype': 'png',
     'dir': 'images'
     },
    {'name': 'grid',
     'render': render_grid,
     'compare': lambda actual, reference: compare_grids(actual, reference, alpha=False),
     'threshold': grid_threshold,
     'filetype': 'json',
     'dir': 'grids'
     }
]

COMPUTE_THRESHOLD = 16

# testcase images are generated on OS X
# so they should exactly match
if platform.uname()[0] == 'Darwin':
    COMPUTE_THRESHOLD = 2

# compare two images and return number of different pixels


def compare(actual, expected, alpha=True):
    im1 = mapnik.Image.open(actual)
    im2 = mapnik.Image.open(expected)
    return im1.compare(im2, COMPUTE_THRESHOLD, alpha)


def compare_grids(actual, expected, threshold=0, alpha=True):
    global errors
    global passed
    im1 = json.loads(open(actual).read())
    im2 = json.loads(open(expected).read())
    # TODO - real diffing
    if not im1['data'] == im2['data']:
        return 99999999
    if not im1['keys'] == im2['keys']:
        return 99999999
    grid1 = im1['grid']
    grid2 = im2['grid']
    # dimensions must be exact
    width1 = len(grid1[0])
    width2 = len(grid2[0])
    if not width1 == width2:
        return 99999999
    height1 = len(grid1)
    height2 = len(grid2)
    if not height1 == height2:
        return 99999999
    diff = 0
    for y in range(0, height1 - 1):
        row1 = grid1[y]
        row2 = grid2[y]
        width = min(len(row1), len(row2))
        for w in range(0, width):
            if row1[w] != row2[w]:
                diff += 1
    return diff

dirname = os.path.join(os.path.dirname(__file__), 'data-visual')


class Reporting:
    DIFF = 1
    NOT_FOUND = 2
    OTHER = 3
    REPLACE = 4

    def __init__(self, quiet, overwrite_failures=False):
        self.quiet = quiet
        self.passed = 0
        self.failed = 0
        self.overwrite_failures = overwrite_failures
        self.errors = [  # (type, actual, expected, diff, message)
        ]

    def result_fail(self, actual, expected, diff):
        self.failed += 1
        if self.quiet:
            if platform.uname()[0] == 'Windows':
                sys.stderr.write('.')
            else:
                sys.stderr.write('\x1b[31m.\x1b[0m')
        else:
            print(
                '\x1b[31m✘\x1b[0m (\x1b[34m%u different pixels\x1b[0m)' %
                diff)

        if self.overwrite_failures:
            self.errors.append((self.REPLACE, actual, expected, diff, None))
            contents = open(actual, 'r').read()
            open(expected, 'wb').write(contents)
        else:
            self.errors.append((self.DIFF, actual, expected, diff, None))

    def result_pass(self, actual, expected, diff):
        self.passed += 1
        if self.quiet:
            if platform.uname()[0] == 'Windows':
                sys.stderr.write('.')
            else:
                sys.stderr.write('\x1b[32m.\x1b[0m')
        else:
            if platform.uname()[0] == 'Windows':
                print('\x1b[32m✓\x1b[0m')
            else:
                print('✓')

    def not_found(self, actual, expected):
        self.failed += 1
        self.errors.append((self.NOT_FOUND, actual, expected, 0, None))
        if self.quiet:
            sys.stderr.write('\x1b[33m.\x1b[0m')
        else:
            print(
                '\x1b[33m?\x1b[0m (\x1b[34mReference file not found, creating\x1b[0m)')
        contents = open(actual, 'r').read()
        open(expected, 'wb').write(contents)

    def other_error(self, expected, message):
        self.failed += 1
        self.errors.append((self.OTHER, None, expected, 0, message))
        if self.quiet:
            sys.stderr.write('\x1b[31m.\x1b[0m')
        else:
            print('\x1b[31m✘\x1b[0m (\x1b[34m%s\x1b[0m)' % message)

    def make_html_item(self, actual, expected, diff):
        item = '''
             <div class="expected">
               <a href="%s">
                 <img src="%s" width="100%s">
               </a>
             </div>
              ''' % (expected, expected, '%')
        item += '<div class="text">%s</div>' % (diff)
        item += '''
             <div class="actual">
               <a href="%s">
                 <img src="%s" width="100%s">
               </a>
             </div>
              ''' % (actual, actual, '%')
        return item

    def summary(self):
        if len(self.errors) == 0:
            print(
                '\nAll %s visual tests passed: \x1b[1;32m✓ \x1b[0m' %
                self.passed)
            return 0
        sortable_errors = []
        print("\nVisual rendering: %s failed / %s passed" %
              (len(self.errors), self.passed))
        for idx, error in enumerate(self.errors):
            if error[0] == self.OTHER:
                print(str(idx +
                          1) +
                      ") \x1b[31mfailure to run test:\x1b[0m %s (\x1b[34m%s\x1b[0m)" %
                      (error[2], error[4]))
            elif error[0] == self.NOT_FOUND:
                print(str(idx + 1) + ") Generating reference image: '%s'" %
                      error[2])
                continue
            elif error[0] == self.DIFF:
                print(
                    str(
                        idx +
                        1) +
                    ") \x1b[34m%s different pixels\x1b[0m:\n\t%s (\x1b[31mactual\x1b[0m)\n\t%s (\x1b[32mexpected\x1b[0m)" %
                    (error[3],
                     error[1],
                        error[2]))
                if '.png' in error[1]:  # ignore grids
                    sortable_errors.append((error[3], error))
            elif error[0] == self.REPLACE:
                print(str(idx +
                          1) +
                      ") \x1b[31mreplaced reference with new version:\x1b[0m %s" %
                      error[2])
        if len(sortable_errors):
            # drop failure results in folder
            vdir = os.path.join(visual_output_dir, 'visual-test-results')
            if not os.path.exists(vdir):
                os.makedirs(vdir)
            html_template = open(
                os.path.join(
                    dirname,
                    'index.html'),
                'r').read()
            name = 'index.html'
            failures_realpath = os.path.join(vdir, name)
            html_out = open(failures_realpath, 'w+')
            sortable_errors.sort(reverse=True)
            html_body = ''
            for item in sortable_errors:
                # copy images into single directory
                actual = item[1][1]
                expected = item[1][2]
                diff = item[0]
                actual_new = os.path.join(vdir, os.path.basename(actual))
                shutil.copy(actual, actual_new)
                expected_new = os.path.join(vdir, os.path.basename(expected))
                shutil.copy(expected, expected_new)
                html_body += self.make_html_item(
                    os.path.relpath(
                        actual_new, vdir), os.path.relpath(
                        expected_new, vdir), diff)
            html_out.write(html_template.replace('{{RESULTS}}', html_body))
            print('View failures by opening %s' % failures_realpath)
        return 1


def render(filename, config, scale_factor, reporting):
    m = mapnik.Map(*config['sizes'][0])

    try:
        mapnik.load_map(m, os.path.join(dirname, "styles", filename), True)

        if not (m.parameters['status'] if (
                'status' in m.parameters) else config['status']):
            return
    except Exception as e:
        if 'Could not create datasource' in str(e) \
           or 'Bad connection' in str(e):
            return m
        reporting.other_error(filename, repr(e))
        return m

    sizes = config['sizes']
    if 'sizes' in m.parameters:
        sizes = [[int(i) for i in size.split(',')]
                 for size in m.parameters['sizes'].split(';')]

    for size in sizes:
        m.width, m.height = size

        if 'bbox' in m.parameters:
            bbox = mapnik.Box2d.from_string(str(m.parameters['bbox']))
            m.zoom_to_box(bbox)
        else:
            m.zoom_all()

        name = filename[0:-4]
        postfix = "%s-%d-%d-%s" % (name, m.width, m.height, scale_factor)
        for renderer in renderers:
            if config.get(renderer['name'], True):
                expected = os.path.join(dirname, renderer['dir'], '%s-%s-reference.%s' %
                                        (postfix, renderer['name'], renderer['filetype']))
                actual = os.path.join(visual_output_dir, '%s-%s.%s' %
                                      (postfix, renderer['name'], renderer['filetype']))
                if not quiet:
                    print("\"%s\" with %s..." % (postfix, renderer['name']))
                try:
                    renderer['render'](m, actual, scale_factor)
                    if not os.path.exists(expected):
                        reporting.not_found(actual, expected)
                    else:
                        diff = renderer['compare'](actual, expected)
                        if diff > renderer['threshold']:
                            reporting.result_fail(actual, expected, diff)
                        else:
                            reporting.result_pass(actual, expected, diff)
                except Exception as e:
                    reporting.other_error(expected, repr(e))
    return m

if __name__ == "__main__":
    if '-q' in sys.argv:
        quiet = True
        sys.argv.remove('-q')
    else:
        quiet = False

    if '--overwrite' in sys.argv:
        overwrite_failures = True
        sys.argv.remove('--overwrite')
    else:
        overwrite_failures = False

    files = None
    if len(sys.argv) > 1:
        files = [name + ".xml" for name in sys.argv[1:]]
    else:
        files = [
            os.path.basename(file) for file in glob.glob(
                os.path.join(
                    dirname,
                    "styles/*.xml"))]

    if not os.path.exists(visual_output_dir):
        os.makedirs(visual_output_dir)

    reporting = Reporting(quiet, overwrite_failures)
    try:
        for filename in files:
            config = dict(defaults)
            for scale_factor in config['scales']:
                m = render(filename, config, scale_factor, reporting)
    except KeyboardInterrupt:
        pass
    sys.exit(reporting.summary())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from binascii import unhexlify

from nose.tools import eq_, raises

import mapnik

from .utilities import run_all


def test_default_constructor():
    f = mapnik.Feature(mapnik.Context(), 1)
    eq_(f is not None, True)


def test_feature_geo_interface():
    ctx = mapnik.Context()
    feat = mapnik.Feature(ctx, 1)
    feat.geometry = mapnik.Geometry.from_wkt('Point (0 0)')
    eq_(feat.__geo_interface__['geometry'], {
        u'type': u'Point', u'coordinates': [0, 0]})


def test_python_extended_constructor():
    context = mapnik.Context()
    context.push('foo')
    context.push('foo')
    f = mapnik.Feature(context, 1)
    wkt = 'POLYGON ((35 10, 10 20, 15 40, 45 45, 35 10),(20 30, 35 35, 30 20, 20 30))'
    f.geometry = mapnik.Geometry.from_wkt(wkt)
    f['foo'] = 'bar'
    eq_(f['foo'], 'bar')
    eq_(f.envelope(), mapnik.Box2d(10.0, 10.0, 45.0, 45.0))
    # reset
    f['foo'] = u"avión"
    eq_(f['foo'], u"avión")
    f['foo'] = 1.4
    eq_(f['foo'], 1.4)
    f['foo'] = True
    eq_(f['foo'], True)


def test_add_geom_wkb():
    # POLYGON ((30 10, 10 20, 20 40, 40 40, 30 10))
    wkb = '010300000001000000050000000000000000003e4000000000000024400000000000002440000000000000344000000000000034400000000000004440000000000000444000000000000044400000000000003e400000000000002440'
    geometry = mapnik.Geometry.from_wkb(unhexlify(wkb))
    if hasattr(geometry, 'is_valid'):
        # Those are only available when python-mapnik has been built with
        # boost >= 1.56.
        eq_(geometry.is_valid(), True)
        eq_(geometry.is_simple(), True)
    eq_(geometry.envelope(), mapnik.Box2d(10.0, 10.0, 40.0, 40.0))
    geometry.correct()
    if hasattr(geometry, 'is_valid'):
        # valid after calling correct
        eq_(geometry.is_valid(), True)


def test_feature_expression_evaluation():
    context = mapnik.Context()
    context.push('name')
    f = mapnik.Feature(context, 1)
    f['name'] = 'a'
    eq_(f['name'], u'a')
    expr = mapnik.Expression("[name]='a'")
    evaluated = expr.evaluate(f)
    eq_(evaluated, True)
    num_attributes = len(f)
    eq_(num_attributes, 1)
    eq_(f.id(), 1)

# https://github.com/mapnik/mapnik/issues/933


def test_feature_expression_evaluation_missing_attr():
    context = mapnik.Context()
    context.push('name')
    f = mapnik.Feature(context, 1)
    f['name'] = u'a'
    eq_(f['name'], u'a')
    expr = mapnik.Expression("[fielddoesnotexist]='a'")
    eq_('fielddoesnotexist' in f, False)
    try:
        expr.evaluate(f)
    except Exception as e:
        eq_("Key does not exist" in str(e), True)
    num_attributes = len(f)
    eq_(num_attributes, 1)
    eq_(f.id(), 1)

# https://github.com/mapnik/mapnik/issues/934


def test_feature_expression_evaluation_attr_with_spaces():
    context = mapnik.Context()
    context.push('name with space')
    f = mapnik.Feature(context, 1)
    f['name with space'] = u'a'
    eq_(f['name with space'], u'a')
    expr = mapnik.Expression("[name with space]='a'")
    eq_(str(expr), "([name with space]='a')")
    eq_(expr.evaluate(f), True)

# https://github.com/mapnik/mapnik/issues/2390


@raises(RuntimeError)
def test_feature_from_geojson():
    ctx = mapnik.Context()
    inline_string = """
    {
         "geometry" : {
            "coordinates" : [ 0,0 ]
            "type" : "Point"
         },
         "type" : "Feature",
         "properties" : {
            "this":"that"
            "known":"nope because missing comma"
         }
    }
    """
    mapnik.Feature.from_geojson(inline_string, ctx)

if __name__ == "__main__":
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import tempfile

from nose.tools import eq_, raises

import mapnik

from .utilities import execution_path, run_all

PYTHON3 = sys.version_info[0] == 3


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_simplest_render():
    m = mapnik.Map(256, 256)
    im = mapnik.Image(m.width, m.height)
    eq_(im.painted(), False)
    eq_(im.is_solid(), True)
    mapnik.render(m, im)
    eq_(im.painted(), False)
    eq_(im.is_solid(), True)
    s = im.tostring()
    if PYTHON3:
        eq_(s, 256 * 256 * b'\x00\x00\x00\x00')
    else:
        eq_(s, 256 * 256 * '\x00\x00\x00\x00')


def test_render_image_to_string():
    im = mapnik.Image(256, 256)
    im.fill(mapnik.Color('black'))
    eq_(im.painted(), False)
    eq_(im.is_solid(), True)
    s = im.tostring()
    if PYTHON3:
        eq_(s, 256 * 256 * b'\x00\x00\x00\xff')
    else:
        eq_(s, 256 * 256 * '\x00\x00\x00\xff')


def test_non_solid_image():
    im = mapnik.Image(256, 256)
    im.fill(mapnik.Color('black'))
    eq_(im.painted(), False)
    eq_(im.is_solid(), True)
    # set one pixel to a different color
    im.set_pixel(0, 0, mapnik.Color('white'))
    eq_(im.painted(), False)
    eq_(im.is_solid(), False)


def test_non_solid_image_view():
    im = mapnik.Image(256, 256)
    im.fill(mapnik.Color('black'))
    view = im.view(0, 0, 256, 256)
    eq_(view.is_solid(), True)
    # set one pixel to a different color
    im.set_pixel(0, 0, mapnik.Color('white'))
    eq_(im.is_solid(), False)
    # view, since it is the exact dimensions of the image
    # should also be non-solid
    eq_(view.is_solid(), False)
    # but not a view that excludes the single diff pixel
    view2 = im.view(1, 1, 256, 256)
    eq_(view2.is_solid(), True)


def test_setting_alpha():
    w, h = 256, 256
    im1 = mapnik.Image(w, h)
    # white, half transparent
    c1 = mapnik.Color('rgba(255,255,255,.5)')
    im1.fill(c1)
    eq_(im1.painted(), False)
    eq_(im1.is_solid(), True)
    # pure white
    im2 = mapnik.Image(w, h)
    c2 = mapnik.Color('rgba(255,255,255,1)')
    im2.fill(c2)
    im2.apply_opacity(c1.a / 255.0)
    eq_(im2.painted(), False)
    eq_(im2.is_solid(), True)
    eq_(len(im1.tostring('png32')), len(im2.tostring('png32')))


def test_render_image_to_file():
    im = mapnik.Image(256, 256)
    im.fill(mapnik.Color('black'))
    if mapnik.has_jpeg():
        im.save('test.jpg')
    im.save('test.png', 'png')
    if os.path.exists('test.jpg'):
        os.remove('test.jpg')
    else:
        return False
    if os.path.exists('test.png'):
        os.remove('test.png')
    else:
        return False


def get_paired_images(w, h, mapfile):
    tmp_map = 'tmp_map.xml'
    m = mapnik.Map(w, h)
    mapnik.load_map(m, mapfile)
    im = mapnik.Image(w, h)
    m.zoom_all()
    mapnik.render(m, im)
    mapnik.save_map(m, tmp_map)
    m2 = mapnik.Map(w, h)
    mapnik.load_map(m2, tmp_map)
    im2 = mapnik.Image(w, h)
    m2.zoom_all()
    mapnik.render(m2, im2)
    os.remove(tmp_map)
    return im, im2


def test_render_from_serialization():
    try:
        im, im2 = get_paired_images(
            100, 100, '../data/good_maps/building_symbolizer.xml')
        eq_(im.tostring('png32'), im2.tostring('png32'))

        im, im2 = get_paired_images(
            100, 100, '../data/good_maps/polygon_symbolizer.xml')
        eq_(im.tostring('png32'), im2.tostring('png32'))
    except RuntimeError as e:
        # only test datasources that we have installed
        if not 'Could not create datasource' in str(e):
            raise RuntimeError(e)


def test_render_points():
    if not mapnik.has_cairo():
        return
    # create and populate point datasource (WGS84 lat-lon coordinates)
    ds = mapnik.MemoryDatasource()
    context = mapnik.Context()
    context.push('Name')
    f = mapnik.Feature(context, 1)
    f['Name'] = 'Westernmost Point'
    f.geometry = mapnik.Geometry.from_wkt('POINT (142.48 -38.38)')
    ds.add_feature(f)

    f = mapnik.Feature(context, 2)
    f['Name'] = 'Southernmost Point'
    f.geometry = mapnik.Geometry.from_wkt('POINT (143.10 -38.60)')
    ds.add_feature(f)

    # create layer/rule/style
    s = mapnik.Style()
    r = mapnik.Rule()
    symb = mapnik.PointSymbolizer()
    symb.allow_overlap = True
    r.symbols.append(symb)
    s.rules.append(r)
    lyr = mapnik.Layer(
        'Places',
        '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs')
    lyr.datasource = ds
    lyr.styles.append('places_labels')
    # latlon bounding box corners
    ul_lonlat = mapnik.Coord(142.30, -38.20)
    lr_lonlat = mapnik.Coord(143.40, -38.80)
    # render for different projections
    projs = {
        'google': '+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0.0 +k=1.0 +units=m +nadgrids=@null +wktext +no_defs +over',
        'latlon': '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs',
        'merc': '+proj=merc +datum=WGS84 +k=1.0 +units=m +over +no_defs',
        'utm': '+proj=utm +zone=54 +datum=WGS84'
    }
    for projdescr in projs:
        m = mapnik.Map(1000, 500, projs[projdescr])
        m.append_style('places_labels', s)
        m.layers.append(lyr)
        dest_proj = mapnik.Projection(projs[projdescr])
        src_proj = mapnik.Projection('+init=epsg:4326')
        tr = mapnik.ProjTransform(src_proj, dest_proj)
        m.zoom_to_box(tr.forward(mapnik.Box2d(ul_lonlat, lr_lonlat)))
        # Render to SVG so that it can be checked how many points are there
        # with string comparison
        svg_file = os.path.join(
            tempfile.gettempdir(),
            'mapnik-render-points-%s.svg' %
            projdescr)
        mapnik.render_to_file(m, svg_file)
        num_points_present = len(list(ds.all_features()))
        with open(svg_file, 'r') as f:
            svg = f.read()
        num_points_rendered = svg.count('<image ')
        eq_(
            num_points_present,
            num_points_rendered,
            "Not all points were rendered (%d instead of %d) at projection %s" %
            (num_points_rendered,
             num_points_present,
             projdescr))


@raises(RuntimeError)
def test_render_with_scale_factor_zero_throws():
    m = mapnik.Map(256, 256)
    im = mapnik.Image(256, 256)
    mapnik.render(m, im, 0.0)


def test_render_with_detector():
    ds = mapnik.MemoryDatasource()
    context = mapnik.Context()
    geojson = '{ "type": "Feature", "geometry": { "type": "Point", "coordinates": [ 0, 0 ] } }'
    ds.add_feature(mapnik.Feature.from_geojson(geojson, context))
    s = mapnik.Style()
    r = mapnik.Rule()
    lyr = mapnik.Layer('point')
    lyr.datasource = ds
    lyr.styles.append('point')
    symb = mapnik.MarkersSymbolizer()
    symb.allow_overlap = False
    r.symbols.append(symb)
    s.rules.append(r)
    m = mapnik.Map(256, 256)
    m.append_style('point', s)
    m.layers.append(lyr)
    m.zoom_to_box(mapnik.Box2d(-180, -85, 180, 85))
    im = mapnik.Image(256, 256)
    mapnik.render(m, im)
    expected_file = './images/support/marker-in-center.png'
    actual_file = '/tmp/' + os.path.basename(expected_file)
    # im.save(expected_file,'png8')
    im.save(actual_file, 'png8')
    actual = mapnik.Image.open(expected_file)
    expected = mapnik.Image.open(expected_file)
    eq_(actual.tostring('png32'),
        expected.tostring('png32'),
        'failed comparing actual (%s) and expected (%s)' % (actual_file,
                                                            expected_file))
    # now render will a collision detector that should
    # block out the placement of this point
    detector = mapnik.LabelCollisionDetector(m)
    eq_(detector.extent(), mapnik.Box2d(-0.0, -0.0, m.width, m.height))
    eq_(detector.extent(), mapnik.Box2d(-0.0, -0.0, 256.0, 256.0))
    eq_(detector.boxes(), [])
    detector.insert(detector.extent())
    eq_(detector.boxes(), [detector.extent()])
    im2 = mapnik.Image(256, 256)
    mapnik.render_with_detector(m, im2, detector)
    expected_file_collision = './images/support/marker-in-center-not-placed.png'
    # im2.save(expected_file_collision,'png8')
    actual_file = '/tmp/' + os.path.basename(expected_file_collision)
    im2.save(actual_file, 'png8')


if 'shape' in mapnik.DatasourceCache.plugin_names():

    def test_render_with_scale_factor():
        m = mapnik.Map(256, 256)
        mapnik.load_map(m, '../data/good_maps/marker-text-line.xml')
        m.zoom_all()
        sizes = [.00001, .005, .1, .899, 1, 1.5, 2, 5, 10, 100]
        for size in sizes:
            im = mapnik.Image(256, 256)
            mapnik.render(m, im, size)
            expected_file = './images/support/marker-text-line-scale-factor-%s.png' % size
            actual_file = '/tmp/' + os.path.basename(expected_file)
            im.save(actual_file, 'png32')
            if os.environ.get('UPDATE'):
                im.save(expected_file, 'png32')
            # we save and re-open here so both png8 images are ready as full
            # color png
            actual = mapnik.Image.open(actual_file)
            expected = mapnik.Image.open(expected_file)
            eq_(actual.tostring('png32'),
                expected.tostring('png32'),
                'failed comparing actual (%s) and expected (%s)' % (actual_file,
                                                                    expected_file))

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import eq_, raises

import mapnik

from .utilities import execution_path, run_all

try:
    import json
except ImportError:
    import simplejson as json


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))

if mapnik.has_grid_renderer():
    def show_grids(name, g1, g2):
        g1_file = '/tmp/mapnik-%s-actual.json' % name
        with open(g1_file, 'w') as f:
            f.write(json.dumps(g1, sort_keys=True))
        g2_file = '/tmp/mapnik-%s-expected.json' % name
        with open(g2_file, 'w') as f:
            f.write(json.dumps(g2, sort_keys=True))
        val = 'JSON does not match  ->\n'
        if g1['grid'] != g2['grid']:
            val += ' X grid does not match\n'
        else:
            val += ' ✓ grid matches\n'
        if g1['data'].keys() != g2['data'].keys():
            val += ' X data does not match\n'
        else:
            val += ' ✓ data matches\n'
        if g1['keys'] != g2['keys']:
            val += ' X keys do not\n'
        else:
            val += ' ✓ keys match\n'
        val += '\n\t%s\n\t%s' % (g1_file, g2_file)
        return val

    def show_grids2(name, g1, g2):
        g2_expected = '../data/grids/mapnik-%s-actual.json' % name
        if not os.path.exists(g2_expected):
            # create test fixture based on actual results
            with open(g2_expected, 'a+') as f:
                f.write(json.dumps(g1, sort_keys=True))
            return
        g1_file = '/tmp/mapnik-%s-actual.json' % name
        with open(g1_file, 'w') as f:
            f.write(json.dumps(g1, sort_keys=True))
        val = 'JSON does not match  ->\n'
        if g1['grid'] != g2['grid']:
            val += ' X grid does not match\n'
        else:
            val += ' ✓ grid matches\n'
        if g1['data'].keys() != g2['data'].keys():
            val += ' X data does not match\n'
        else:
            val += ' ✓ data matches\n'
        if g1['keys'] != g2['keys']:
            val += ' X keys do not\n'
        else:
            val += ' ✓ keys match\n'
        val += '\n\t%s\n\t%s' % (g1_file, g2_expected)
        return val

    # previous rendering using agg ellipse directly
    grid_correct_new = {
        "data": {
            "North East": {
                "Name": "North East"},
            "North West": {
                "Name": "North West"},
            "South East": {
                "Name": "South East"},
            "South West": {
                "Name": "South West"}},
        "grid": [
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          !!                                  ##                ",
            "         !!!                                 ###                ",
            "          !!                                  ##                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          $$                                  %%                ",
            "         $$$                                  %%                ",
            "          $$                                  %%                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                "],
        "keys": [
            "",
            "North West",
            "North East",
            "South West",
            "South East"]}

    # newer rendering using svg
    grid_correct_new2 = {
        "data": {
            "North East": {
                "Name": "North East"},
            "North West": {
                "Name": "North West"},
            "South East": {
                "Name": "South East"},
            "South West": {
                "Name": "South West"}},
        "grid": [
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          !!                                  ##                ",
            "         !!!                                 ###                ",
            "          !!                                  ##                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          $$                                  %%                ",
            "         $$$                                  %%                ",
            "          $$                                  %%                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                "],
        "keys": [
            "",
            "North West",
            "North East",
            "South West",
            "South East"]}

    grid_correct_new3 = {
        "data": {
            "North East": {
                "Name": "North East"},
            "North West": {
                "Name": "North West"},
            "South East": {
                "Name": "South East"},
            "South West": {
                "Name": "South West"}},
        "grid": [
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          !!                                  ##                ",
            "         !!!                                 ###                ",
            "          !!                                  ##                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          $$                                  %%                ",
            "         $$$                                  %%                ",
            "          $                                   %%                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                "],
        "keys": [
            "",
            "North West",
            "North East",
            "South West",
            "South East"]}

    def resolve(grid, row, col):
        """ Resolve the attributes for a given pixel in a grid.
        """
        row = grid['grid'][row]
        utf_val = row[col]
        # http://docs.python.org/library/functions.html#ord
        codepoint = ord(utf_val)
        if (codepoint >= 93):
            codepoint -= 1
        if (codepoint >= 35):
            codepoint -= 1
        codepoint -= 32
        key = grid['keys'][codepoint]
        return grid['data'].get(key)

    def create_grid_map(width, height, sym):
        ds = mapnik.MemoryDatasource()
        context = mapnik.Context()
        context.push('Name')
        f = mapnik.Feature(context, 1)
        f['Name'] = 'South East'
        f.geometry = mapnik.Geometry.from_wkt('POINT (143.10 -38.60)')
        ds.add_feature(f)

        f = mapnik.Feature(context, 2)
        f['Name'] = 'South West'
        f.geometry = mapnik.Geometry.from_wkt('POINT (142.48 -38.60)')
        ds.add_feature(f)

        f = mapnik.Feature(context, 3)
        f['Name'] = 'North West'
        f.geometry = mapnik.Geometry.from_wkt('POINT (142.48 -38.38)')
        ds.add_feature(f)

        f = mapnik.Feature(context, 4)
        f['Name'] = 'North East'
        f.geometry = mapnik.Geometry.from_wkt('POINT (143.10 -38.38)')
        ds.add_feature(f)
        s = mapnik.Style()
        r = mapnik.Rule()
        sym.allow_overlap = True
        r.symbols.append(sym)
        s.rules.append(r)
        lyr = mapnik.Layer('Places')
        lyr.datasource = ds
        lyr.styles.append('places_labels')
        m = mapnik.Map(width, height)
        m.append_style('places_labels', s)
        m.layers.append(lyr)
        return m

    def test_render_grid():
        """ test render_grid method"""
        width, height = 256, 256
        sym = mapnik.MarkersSymbolizer()
        sym.width = mapnik.Expression('10')
        sym.height = mapnik.Expression('10')
        m = create_grid_map(width, height, sym)
        ul_lonlat = mapnik.Coord(142.30, -38.20)
        lr_lonlat = mapnik.Coord(143.40, -38.80)
        m.zoom_to_box(mapnik.Box2d(ul_lonlat, lr_lonlat))

        # new method
        grid = mapnik.Grid(m.width, m.height, key='Name')
        mapnik.render_layer(m, grid, layer=0, fields=['Name'])
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1, grid_correct_new3, show_grids(
            'new-markers', utf1, grid_correct_new3))

        # check a full view is the same as a full image
        grid_view = grid.view(0, 0, width, height)
        # for kicks check at full res too
        utf3 = grid.encode('utf', resolution=1)
        utf4 = grid_view.encode('utf', resolution=1)
        eq_(utf3['grid'], utf4['grid'])
        eq_(utf3['keys'], utf4['keys'])
        eq_(utf3['data'], utf4['data'])

        eq_(resolve(utf4, 0, 0), None)

        # resolve some center points in the
        # resampled view
        utf5 = grid_view.encode('utf', resolution=4)
        eq_(resolve(utf5, 25, 10), {"Name": "North West"})
        eq_(resolve(utf5, 25, 46), {"Name": "North East"})
        eq_(resolve(utf5, 38, 10), {"Name": "South West"})
        eq_(resolve(utf5, 38, 46), {"Name": "South East"})

    grid_feat_id = {
        'keys': [
            '',
            '3',
            '4',
            '2',
            '1'],
        'data': {
            '1': {
                'Name': 'South East'},
            '3': {
                'Name': u'North West'},
            '2': {
                'Name': 'South West'},
            '4': {
                'Name': 'North East'}},
        'grid': [
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '          !!                                  ##                ',
            '         !!!                                 ###                ',
            '          !!                                  ##                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '         $$$                                  %%                ',
            '         $$$                                 %%%                ',
            '          $$                                  %%                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ',
            '                                                                ']}

    grid_feat_id2 = {
        "data": {
            "1": {
                "Name": "South East"},
            "2": {
                "Name": "South West"},
            "3": {
                "Name": "North West"},
            "4": {
                "Name": "North East"}},
        "grid": [
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          !!                                  ##                ",
            "         !!!                                 ###                ",
            "          !!                                  ##                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          $$                                  %%                ",
            "         $$$                                  %%                ",
            "          $$                                  %%                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                "],
        "keys": [
            "",
            "3",
            "4",
            "2",
            "1"]}

    grid_feat_id3 = {
        "data": {
            "1": {
                "Name": "South East",
                "__id__": 1},
            "2": {
                "Name": "South West",
                "__id__": 2},
            "3": {
                "Name": "North West",
                "__id__": 3},
            "4": {
                "Name": "North East",
                "__id__": 4}},
        "grid": [
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          !!                                  ##                ",
            "         !!!                                 ###                ",
            "          !!                                  ##                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "          $$                                  %%                ",
            "         $$$                                  %%                ",
            "          $                                   %%                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                "],
        "keys": [
            "",
            "3",
            "4",
            "2",
            "1"]}

    def test_render_grid3():
        """ test using feature id"""
        width, height = 256, 256
        sym = mapnik.MarkersSymbolizer()
        sym.width = mapnik.Expression('10')
        sym.height = mapnik.Expression('10')
        m = create_grid_map(width, height, sym)
        ul_lonlat = mapnik.Coord(142.30, -38.20)
        lr_lonlat = mapnik.Coord(143.40, -38.80)
        m.zoom_to_box(mapnik.Box2d(ul_lonlat, lr_lonlat))

        grid = mapnik.Grid(m.width, m.height, key='__id__')
        mapnik.render_layer(m, grid, layer=0, fields=['__id__', 'Name'])
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1, grid_feat_id3, show_grids('id-markers', utf1, grid_feat_id3))
        # check a full view is the same as a full image
        grid_view = grid.view(0, 0, width, height)
        # for kicks check at full res too
        utf3 = grid.encode('utf', resolution=1)
        utf4 = grid_view.encode('utf', resolution=1)
        eq_(utf3['grid'], utf4['grid'])
        eq_(utf3['keys'], utf4['keys'])
        eq_(utf3['data'], utf4['data'])

        eq_(resolve(utf4, 0, 0), None)

        # resolve some center points in the
        # resampled view
        utf5 = grid_view.encode('utf', resolution=4)
        eq_(resolve(utf5, 25, 10), {"Name": "North West", "__id__": 3})
        eq_(resolve(utf5, 25, 46), {"Name": "North East", "__id__": 4})
        eq_(resolve(utf5, 38, 10), {"Name": "South West", "__id__": 2})
        eq_(resolve(utf5, 38, 46), {"Name": "South East", "__id__": 1})

    def gen_grid_for_id(pixel_key):
        ds = mapnik.MemoryDatasource()
        context = mapnik.Context()
        context.push('Name')
        f = mapnik.Feature(context, pixel_key)
        f['Name'] = str(pixel_key)
        f.geometry = mapnik.Geometry.from_wkt(
            'POLYGON ((0 0, 0 256, 256 256, 256 0, 0 0))')
        ds.add_feature(f)
        s = mapnik.Style()
        r = mapnik.Rule()
        symb = mapnik.PolygonSymbolizer()
        r.symbols.append(symb)
        s.rules.append(r)
        lyr = mapnik.Layer('Places')
        lyr.datasource = ds
        lyr.styles.append('places_labels')
        width, height = 256, 256
        m = mapnik.Map(width, height)
        m.append_style('places_labels', s)
        m.layers.append(lyr)
        m.zoom_all()
        grid = mapnik.Grid(m.width, m.height, key='__id__')
        mapnik.render_layer(m, grid, layer=0, fields=['__id__', 'Name'])
        return grid

    def test_negative_id():
        grid = gen_grid_for_id(-1)
        eq_(grid.get_pixel(128, 128), -1)
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1['keys'], ['-1'])

    def test_32bit_int_id():
        int32 = 2147483647
        grid = gen_grid_for_id(int32)
        eq_(grid.get_pixel(128, 128), int32)
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1['keys'], [str(int32)])
        max_neg = -(int32)
        grid = gen_grid_for_id(max_neg)
        eq_(grid.get_pixel(128, 128), max_neg)
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1['keys'], [str(max_neg)])

    def test_64bit_int_id():
        int64 = 0x7FFFFFFFFFFFFFFF
        grid = gen_grid_for_id(int64)
        eq_(grid.get_pixel(128, 128), int64)
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1['keys'], [str(int64)])
        max_neg = -(int64)
        grid = gen_grid_for_id(max_neg)
        eq_(grid.get_pixel(128, 128), max_neg)
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1['keys'], [str(max_neg)])

    def test_id_zero():
        grid = gen_grid_for_id(0)
        eq_(grid.get_pixel(128, 128), 0)
        utf1 = grid.encode('utf', resolution=4)
        eq_(utf1['keys'], ['0'])

    line_expected = {
        "keys": [
            "",
            "1"],
        "data": {
            "1": {
                "Name": "1"}},
        "grid": [
            "                                                               !",
            "                                                            !!  ",
            "                                                         !!     ",
            "                                                      !!        ",
            "                                                   !!           ",
            "                                                !!              ",
            "                                             !!                 ",
            "                                          !!                    ",
            "                                       !!                       ",
            "                                    !!                          ",
            "                                 !!                             ",
            "                              !!                                ",
            "                           !!                                   ",
            "                        !!                                      ",
            "                     !!                                         ",
            "                  !!                                            ",
            "               !!                                               ",
            "            !!                                                  ",
            "         !!                                                     ",
            "      !!                                                        ",
            "   !!                                                           ",
            "!!                                                              ",
            " !                                                              ",
            "  !                                                             ",
            "   !                                                            ",
            "    !                                                           ",
            "     !                                                          ",
            "      !                                                         ",
            "       !                                                        ",
            "        !                                                       ",
            "         !                                                      ",
            "          !                                                     ",
            "           !                                                    ",
            "            !                                                   ",
            "             !                                                  ",
            "              !                                                 ",
            "               !                                                ",
            "                !                                               ",
            "                 !                                              ",
            "                  !                                             ",
            "                   !                                            ",
            "                    !                                           ",
            "                     !                                          ",
            "                      !                                         ",
            "                       !                                        ",
            "                        !                                       ",
            "                         !                                      ",
            "                          !                                     ",
            "                           !                                    ",
            "                            !                                   ",
            "                             !                                  ",
            "                              !                                 ",
            "                               !                                ",
            "                                !                               ",
            "                                 !                              ",
            "                                  !                             ",
            "                                   !                            ",
            "                                    !                           ",
            "                                     !                          ",
            "                                      !                         ",
            "                                       !                        ",
            "                                        !                       ",
            "                                         !                      ",
            "                                          !                     "]}

    def test_line_rendering():
        ds = mapnik.MemoryDatasource()
        context = mapnik.Context()
        context.push('Name')
        pixel_key = 1
        f = mapnik.Feature(context, pixel_key)
        f['Name'] = str(pixel_key)
        f.geometry = mapnik.Geometry.from_wkt(
            'LINESTRING (30 10, 10 30, 40 40)')
        ds.add_feature(f)
        s = mapnik.Style()
        r = mapnik.Rule()
        symb = mapnik.LineSymbolizer()
        r.symbols.append(symb)
        s.rules.append(r)
        lyr = mapnik.Layer('Places')
        lyr.datasource = ds
        lyr.styles.append('places_labels')
        width, height = 256, 256
        m = mapnik.Map(width, height)
        m.append_style('places_labels', s)
        m.layers.append(lyr)
        m.zoom_all()
        # mapnik.render_to_file(m,'test.png')
        grid = mapnik.Grid(m.width, m.height, key='__id__')
        mapnik.render_layer(m, grid, layer=0, fields=['Name'])
        utf1 = grid.encode()
        eq_(utf1, line_expected, show_grids('line', utf1, line_expected))

    point_expected = {
        "data": {
            "1": {
                "Name": "South East"},
            "2": {
                "Name": "South West"},
            "3": {
                "Name": "North West"},
            "4": {
                "Name": "North East"}},
        "grid": [
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "         !!!!                                ####               ",
            "         !!!!                                ####               ",
            "         !!!!                                ####               ",
            "         !!!!                                ####               ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "         $$$$                                %%%%               ",
            "         $$$$                                %%%%               ",
            "         $$$$                                %%%%               ",
            "         $$$$                                %%%%               ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                ",
            "                                                                "],
        "keys": [
            "",
            "3",
            "4",
            "2",
            "1"]}

    def test_point_symbolizer_grid():
        width, height = 256, 256
        sym = mapnik.PointSymbolizer()
        sym.file = '../data/images/dummy.png'
        m = create_grid_map(width, height, sym)
        ul_lonlat = mapnik.Coord(142.30, -38.20)
        lr_lonlat = mapnik.Coord(143.40, -38.80)
        m.zoom_to_box(mapnik.Box2d(ul_lonlat, lr_lonlat))
        grid = mapnik.Grid(m.width, m.height)
        mapnik.render_layer(m, grid, layer=0, fields=['Name'])
        utf1 = grid.encode()
        eq_(utf1, point_expected, show_grids('point-sym', utf1, point_expected))

    test_point_symbolizer_grid.requires_data = True

    # should throw because this is a mis-usage
    # https://github.com/mapnik/mapnik/issues/1325
    @raises(RuntimeError)
    def test_render_to_grid_multiple_times():
        # create map with two layers
        m = mapnik.Map(256, 256)
        s = mapnik.Style()
        r = mapnik.Rule()
        sym = mapnik.MarkersSymbolizer()
        sym.allow_overlap = True
        r.symbols.append(sym)
        s.rules.append(r)
        m.append_style('points', s)

        # NOTE: we use a csv datasource here
        # because the memorydatasource fails silently for
        # queries requesting fields that do not exist in the datasource
        ds1 = mapnik.Datasource(**{"type": "csv", "inline": '''
          wkt,Name
          "POINT (143.10 -38.60)",South East'''})
        lyr1 = mapnik.Layer('One')
        lyr1.datasource = ds1
        lyr1.styles.append('points')
        m.layers.append(lyr1)

        ds2 = mapnik.Datasource(**{"type": "csv", "inline": '''
          wkt,Value
          "POINT (142.48 -38.60)",South West'''})
        lyr2 = mapnik.Layer('Two')
        lyr2.datasource = ds2
        lyr2.styles.append('points')
        m.layers.append(lyr2)

        ul_lonlat = mapnik.Coord(142.30, -38.20)
        lr_lonlat = mapnik.Coord(143.40, -38.80)
        m.zoom_to_box(mapnik.Box2d(ul_lonlat, lr_lonlat))
        grid = mapnik.Grid(m.width, m.height)
        mapnik.render_layer(m, grid, layer=0, fields=['Name'])
        # should throw right here since Name will be a property now on the `grid` object
        # and it is not found on the second layer
        mapnik.render_layer(m, grid, layer=1, fields=['Value'])
        grid.encode()

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
import os
import sys
from itertools import groupby

from nose.tools import eq_, raises

import mapnik

from .utilities import execution_path, run_all

PYTHON3 = sys.version_info[0] == 3
if PYTHON3:
    xrange = range


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_that_datasources_exist():
    if len(mapnik.DatasourceCache.plugin_names()) == 0:
        print('***NOTICE*** - no datasource plugins have been loaded')

# adapted from raster_symboliser_test#test_dataraster_query_point


@raises(RuntimeError)
def test_vrt_referring_to_missing_files():
    srs = '+init=epsg:32630'
    if 'gdal' in mapnik.DatasourceCache.plugin_names():
        lyr = mapnik.Layer('dataraster')
        lyr.datasource = mapnik.Gdal(
            file='../data/raster/missing_raster.vrt',
            band=1,
        )
        lyr.srs = srs
        _map = mapnik.Map(256, 256, srs)
        _map.layers.append(lyr)

        # center of extent of raster
        x, y = 556113.0, 4381428.0  # center of extent of raster

        _map.zoom_all()

        # Fancy stuff to supress output of error
        # open 2 fds
        null_fds = [os.open(os.devnull, os.O_RDWR) for x in xrange(2)]
        # save the current file descriptors to a tuple
        save = os.dup(1), os.dup(2)
        # put /dev/null fds on 1 and 2
        os.dup2(null_fds[0], 1)
        os.dup2(null_fds[1], 2)

        # *** run the function ***
        try:
            # Should RuntimeError here
            list(_map.query_point(0, x, y))
        finally:
            # restore file descriptors so I can print the results
            os.dup2(save[0], 1)
            os.dup2(save[1], 2)
            # close the temporary fds
            os.close(null_fds[0])
            os.close(null_fds[1])


def test_field_listing():
    if 'shape' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Shapefile(file='../data/shp/poly.shp')
        fields = ds.fields()
        eq_(fields, ['AREA', 'EAS_ID', 'PRFEDEA'])
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Polygon)
        eq_(desc['name'], 'shape')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')


def test_total_feature_count_shp():
    if 'shape' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Shapefile(file='../data/shp/poly.shp')
        features = ds.all_features()
        num_feats = len(list(features))
        eq_(num_feats, 10)


def test_total_feature_count_json():
    if 'ogr' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Ogr(file='../data/json/points.geojson', layer_by_index=0)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Point)
        eq_(desc['name'], 'ogr')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')
        features = ds.all_features()
        num_feats = len(list(features))
        eq_(num_feats, 5)


def test_sqlite_reading():
    if 'sqlite' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.SQLite(
            file='../data/sqlite/world.sqlite',
            table_by_index=0)
        desc = ds.describe()
        eq_(desc['geometry_type'], mapnik.DataGeometryType.Polygon)
        eq_(desc['name'], 'sqlite')
        eq_(desc['type'], mapnik.DataType.Vector)
        eq_(desc['encoding'], 'utf-8')
        features = ds.all_features()
        num_feats = len(list(features))
        eq_(num_feats, 245)


def test_reading_json_from_string():
    with open('../data/json/points.geojson', 'r') as f:
        json = f.read()
    if 'ogr' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Ogr(file=json, layer_by_index=0)
        features = ds.all_features()
        num_feats = len(list(features))
        eq_(num_feats, 5)


def test_feature_envelope():
    if 'shape' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Shapefile(file='../data/shp/poly.shp')
        features = ds.all_features()
        for feat in features:
            env = feat.envelope()
            contains = ds.envelope().contains(env)
            eq_(contains, True)
            intersects = ds.envelope().contains(env)
            eq_(intersects, True)


def test_feature_attributes():
    if 'shape' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Shapefile(file='../data/shp/poly.shp')
        features = list(ds.all_features())
        feat = features[0]
        attrs = {'PRFEDEA': u'35043411', 'EAS_ID': 168, 'AREA': 215229.266}
        eq_(feat.attributes, attrs)
        eq_(ds.fields(), ['AREA', 'EAS_ID', 'PRFEDEA'])
        eq_(ds.field_types(), ['float', 'int', 'str'])


def test_ogr_layer_by_sql():
    if 'ogr' in mapnik.DatasourceCache.plugin_names():
        ds = mapnik.Ogr(file='../data/shp/poly.shp',
                        layer_by_sql='SELECT * FROM poly WHERE EAS_ID = 168')
        features = ds.all_features()
        num_feats = len(list(features))
        eq_(num_feats, 1)


def test_hit_grid():

    def rle_encode(l):
        """ encode a list of strings with run-length compression """
        return ["%d:%s" % (len(list(group)), name)
                for name, group in groupby(l)]

    m = mapnik.Map(256, 256)
    try:
        mapnik.load_map(m, '../data/good_maps/agg_poly_gamma_map.xml')
        m.zoom_all()
        join_field = 'NAME'
        fg = []  # feature grid
        for y in xrange(0, 256, 4):
            for x in xrange(0, 256, 4):
                featureset = m.query_map_point(0, x, y)
                added = False
                for feature in featureset:
                    fg.append(feature[join_field])
                    added = True
                if not added:
                    fg.append('')
        hit_list = '|'.join(rle_encode(fg))
        eq_(hit_list[:16], '730:|2:Greenland')
        eq_(hit_list[-12:], '1:Chile|812:')
    except RuntimeError as e:
        # only test datasources that we have installed
        if not 'Could not create datasource' in str(e):
            raise RuntimeError(str(e))


if __name__ == '__main__':
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from nose.tools import assert_almost_equal, eq_, raises

import mapnik

from .utilities import execution_path, run_all


def setup():
    # All of the paths used are relative, if we run the tests
    # from another directory we need to chdir()
    os.chdir(execution_path('.'))


def test_query_init():
    bbox = (-180, -90, 180, 90)
    query = mapnik.Query(mapnik.Box2d(*bbox))
    r = query.resolution
    assert_almost_equal(r[0], 1.0, places=7)
    assert_almost_equal(r[1], 1.0, places=7)
    # https://github.com/mapnik/mapnik/issues/1762
    eq_(query.property_names, [])
    query.add_property_name('migurski')
    eq_(query.property_names, ['migurski'])

# Converting *from* tuples *to* resolutions is not yet supported


@raises(TypeError)
def test_query_resolution():
    bbox = (-180, -90, 180, 90)
    init_res = (4.5, 6.7)
    query = mapnik.Query(mapnik.Box2d(*bbox), init_res)
    r = query.resolution
    assert_almost_equal(r[0], init_res[0], places=7)
    assert_almost_equal(r[1], init_res[1], places=7)

if __name__ == "__main__":
    setup()
    exit(run_all(eval(x) for x in dir() if x.startswith("test_")))



#!/usr/bin/env python

import getopt
import os
import sys


from python_tests.utilities import TodoPlugin

try:
    import nose
except ImportError as e:
    sys.stderr.write(
        "Unable to run python tests: the third party 'nose' module is required"
        "\nTo install 'nose' do:"
        "\n\tsudo pip install nose (or on debian systems: "
        "apt-get install python-nose): %s\n" % e)
    sys.exit(1)
else:
    from nose.plugins.doctests import Doctest


def usage():
    print("test.py -h | --help")
    print("test.py [-q | -v] [-p | --prefix <path>]")


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hvqp:", ["help", "prefix="])
    except getopt.GetoptError as err:
        print(str(err))
        usage()
        sys.exit(2)

    prefix = None
    verbose = False
    quiet = False

    for o, a in opts:
        if o == "-q":
            quiet = True
        elif o == "-v":
            verbose = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-p", "--prefix"):
            prefix = a
        else:
            assert False, "Unhandled option"

    if quiet and verbose:
        usage()
        sys.exit(2)

    if prefix:
        # Allow python to find libraries for testing on the buildbot
        sys.path.insert(
            0,
            os.path.join(
                prefix,
                "lib/python%s/site-packages" %
                sys.version[
                    :3]))

    import mapnik

    if not quiet:
        print("- mapnik path: %s" % mapnik.__file__)
        if hasattr(mapnik, '_mapnik'):
            print("- _mapnik.so path: %s" % mapnik._mapnik.__file__)
        if hasattr(mapnik, 'inputpluginspath'):
            print("- Input plugins path: %s" % mapnik.inputpluginspath)
        if 'MAPNIK_INPUT_PLUGINS_DIRECTORY' in os.environ:
            print("- MAPNIK_INPUT_PLUGINS_DIRECTORY env: %s" %
                  os.environ.get('MAPNIK_INPUT_PLUGINS_DIRECTORY'))
        if hasattr(mapnik, 'fontscollectionpath'):
            print("- Font path: %s" % mapnik.fontscollectionpath)
        if 'MAPNIK_FONT_DIRECTORY' in os.environ:
            print(
                "- MAPNIK_FONT_DIRECTORY env: %s" %
                os.environ.get('MAPNIK_FONT_DIRECTORY'))
        print('')
        print("- Running nosetests:")
        print('')

    argv = [
        __file__,
        '--exe',
        '--with-todo',
        '--with-doctest',
        '--doctest-tests']

    if not quiet:
        argv.append('-v')

    if verbose:
        # 3 * '-v' gets us debugging information from nose
        argv.append('-v')
        argv.append('-v')

    dirname = os.path.dirname(sys.argv[0])
    argv.extend(['-w', os.path.join(dirname, 'python_tests')])

    if not nose.run(argv=argv, plugins=[TodoPlugin(), Doctest()]):
        sys.exit(1)
    else:
        sys.exit(0)

if __name__ == "__main__":
    main()
